# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2020, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Eason
# @version  1.0.0
# @brief    命令
# @address
#
#  History:
#      <author>     <time>     <version>   <desc>
#       Eason      2010-05-13     1.0.0   命令
#############################################################

import time
import logging
import json
import hrse.hrandom as hrandom
import hrse.mqtt as mqtt

logger = logging.getLogger(__name__)


__version__ = "1.0.0"


__all__ = [
    "init",
    "send",
    "register",
    "unregister",
]


"""
# 常量
"""
HRE_TOPIC = "/hre/command"


"""
# 枚举
"""
class CommandType(object):
    """
    命令类型
    """
    # 发送命令
    SEND = "send"
    # 响应数据
    RESP = "resp"


"""
# 全局变量
"""
# 设备ID
__id = None

# 发送列表
__sendCommands = []

# 消息回调列表
__receiveCallbacks = []


def __mqttReceiveCallback(msg):
    """
    MQTT收到消息回调
    :param msg: 消息
                payload
                    srcId: 发送方ID
                    destId： 目标ID
                    id: 消息id
                    type: 类型
                    name: 名称
                    data: 数据体
    :return:
    """
    global HRE_TOPIC
    global __id
    global __sendCommands
    global __receiveCallbacks

    command = json.loads(msg.payload)

    # 目标地址必须是本地址才处理
    if __id == command.get('destId'):
        # 收到发送命令
        if command.get('type') == CommandType.SEND:
            srcId = command.get('srcId')
            name = command.get('name')
            id = command.get('id')
            data = command.get('data')

            def success(data):
                """
                成功回调
                :param data: 回调数据
                """
                # 响应数据到发送方
                mqtt.send((HRE_TOPIC + srcId), payload=json.dumps({
                    'srcId': __id,
                    'destId': srcId,
                    'id': id,
                    'type': CommandType.RESP,
                    'name': name,
                    'data': data,
                }))

            # 执行接收回调
            for receiveCallback in __receiveCallbacks:
                if receiveCallback.get('name') == name:
                    receiveCallback.get('callback')(data, {
                        'success': success
                    })

        # 收到响应数据
        elif command.get('type') == CommandType.RESP:
            for i in range(len(__sendCommands) - 1, -1, -1):
                # 通过id找到发送的命令
                if __sendCommands[i].get('id') == command.get('id'):
                    options = __sendCommands[i].get('options')

                    # 回调数据
                    options.get('success')  and options.get('success')(command.get('data'))
                    options.get('complete') and options.get('complete')()

                    # 删除命令
                    del __sendCommands[i]
        else:
            logger.error("收到不支持的命令：{}".format(str(command)))


def __generateSendCommandId():
    """
    生成发送命令ID
    :return:
    """
    global __sendCommands

    newId = 0
    while(True):
        newId += 1

        isUnique = True
        for sendCommand in __sendCommands:
            if sendCommand.get('id') is newId:
                isUnique = False

        if isUnique:
            return newId


def init(id, options={}):
    """
    初始化
    :param id: ID标识
    :param options:
    :return:
    """
    global HRE_TOPIC
    global __id
    __id = id

    # 注册监听该id的主题
    mqtt.register((HRE_TOPIC + __id), __mqttReceiveCallback)
    mqttClient = mqtt.getClinet()
    if mqttClient is not None:
        mqttClient.on_disconnect = on_disconnect


def send(id, name, data, options={}):
    """
    发送命令
    :param id: 目标ID
    :param name: 名称
    :param data: 数据
    :param options:
                success
                fail
                complete
    :return:
    """
    global HRE_TOPIC
    global __sendCommands

    command = {
        'srcId': __id,
        'destId': id,
        'id': __generateSendCommandId(),
        'type': CommandType.SEND,
        'name': name,
        'data': data,
    }

    # 发送命令到目标主题
    mqtt.send((HRE_TOPIC + id), payload=json.dumps(command))

    command['options'] = options

    __sendCommands.append(command)


def register(name, callback):
    """
    注册回调
    :param name: 主题
    :param callback: 回调
                data: 数据
                options:
                    success: 响应回调
    :return: 取消注册方法
    """
    global __receiveCallbacks

    __receiveCallbacks.append({
        'name': name,
        'callback': callback
    })

    def _unregister():
        unregister(name=name, callback=callback)

    return _unregister


def unregister(name, callback):
    """
    取消注册回调
    :param name: 主题
    :param callback: 回调
    :return:
    """
    global __receiveCallbacks

    for i in range(len(__receiveCallbacks) - 1, -1, -1):
        if name == __receiveCallbacks[i].get('name') and callback == __receiveCallbacks[i].get('callback'):
            del __receiveCallbacks[i]


def checkConn():
    """
    检查连接状态
    :return:
    """
    global __id
    client = mqtt.getClinet()
    if client is not None:
        connStatus = client.is_connected()
        if connStatus is False:
            # 重新连接mqtt客户端
            client.reconnect()
            logger.info("reconnect mqtt client")

            # 重新订阅消息
            if __id is not None:
                init(__id)
                logger.info("init mqtt")
            else:
                logger.error("__id为空，无法初始化")
    else:
        logger.info("获取mqtt客户端失败")


def on_disconnect(mqttc, userdata, rc):
    """
    断开连接后重连
    :param mqttc:      客户端
    :param userdata:   用户数据
    :param rc:         结果值
    :return:
    """
    global __id
    if rc != 0:
        logger.info(" Unexpected disconnection")
        try:
            if __id is None:
                __id = 'hrse0001'
            clientid = "o_{}".format(hrandom.generate(12))
            username = "tx2_o_n_{}_{}".format('hrse0001', hrandom.generate(4))
            mqtt.init(clientid=clientid, username=username)
            logger.info("init mqtt success")
            client = mqtt.getClinet()
            # client.unsubscribe(HRE_TOPIC + __id)
            client.subscribe(HRE_TOPIC + __id, 2)
            logger.info("subscribe topic success")
        except Exception as err:
            logger.info("Error in Retrying to Connect with Broker")
            logger.error(err)


if __name__ == "__main__":
    deviceid = "hrs0001"
    clientid = "o_{}".format(hrandom.generate(12))
    username = "tx2_o_n_{}_{}".format(deviceid, hrandom.generate(4))

    mqtt.init(clientid=clientid, username=username)

    # 初始化Command
    init("hrs0001")

    # 接收回调
    def receiveCb(data, options={}):
        logger.error("receive data={}".format(str(data)))
        options.get('success') and options.get('success')({
            'hello': 'Eason'
        })

    register("hello", receiveCb)


    def sendCb(data):
        logger.error("response data={}".format(str(data)))

    send("hrs0001", "hello", {
        'message': "USServer to Server"
    }, {
        'success': sendCb
    })

    while(True):
        time.sleep(1)

    pass
