# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2020, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Levi
# @version  1.0.0
# @brief    GEE数据传输
# @address  GEE官网:     https://earthengine.google.com/
#           GitHub:     https://github.com/google/earthengine-api
#
#  History:
#      <author>     <time>     <version>   <desc>
#       Levi      2010-06-04     1.0.0   GEE数据传输
#############################################################
import datetime
import logging
import os
import threading
import time

import ee
from ee.batch import Export

from hrse import gcs, hrandom, mqtt, command, drive
from hrse.core import oss
from hrse.hlib.enum import CommandEnum


__version__ = "1.0.0"


__all__ = [
    'init',
    'deinit',
    
    'uploadToAssets',
    'downloadFromAssets',

    'uploadToGcs',
    'downloadFromGcs',

    'downloadFromDatasets',
    'downloadImageFile',

    'queryRecentDate',
]


"""
# 常量
"""
# 最大下载影像数量
MAX_DOWNLOAD_IMAGE_COUNT = 2000


"""
# 枚举
"""
class DatasetsPathType(object):
    SENTINEL_LEVEL_1C = "COPERNICUS/S2"
    SENTINEL_LEVEL_2A = "COPERNICUS/S2_SR"
    LANDSAT8_SR_T1 = "LANDSAT/LC08/C01/T1_SR"
    LANDSAT8_TOA_T1 = "LANDSAT/LC08/C01/T1_TOA"
    LANDSAT8_8DAY_NDVI = "LANDSAT/LC08/C01/T1_8DAY_NDVI"
    LANDSAT8_8DAY_EVI = "LANDSAT/LC08/C01/T1_8DAY_EVI"
    LANDSAT8_8DAY_NDWI = "LANDSAT/LC08/C01/T1_8DAY_NDWI"
    LANDSAT8_8DAY_NDSI = "LANDSAT/LC08/C01/T1_8DAY_NDSI"
    MODIS_MOD11A1 = "MODIS/006/MOD11A1"
    MODIS_MOD11A2 = "MODIS/006/MOD11A2"


"""
# 全局变量
"""
# 日志对象
logger = logging.getLogger(__name__)

# oss连接信息
__ossInfo = {}

# 设备id
__deviceid = 'hrse0002'

# 美国服务器id
__usserverId = 'hrse0001'


def init(options={}):
    """
    初始化配置
    :param options:
               host
               port
               username
               password
               deviceid
    :return:
    """
    global __deviceid
    global __ossInfo

    oss.init()
    getOssInfo()

    if options.get("deviceid") is not None:
        __deviceid = options.get("deviceid")

    __initMqtt(__deviceid)


def deinit(options={}):
    global __ossInfo

    __ossInfo = None


def uploadToAssets(localPath, assetsPath, options={}):
    """
    上传本地文件到GEE资产管理
    :param localPath:      本地文件路径
    :param assetsPath:     资产路径
    :param options:
               success
                   data
               fail
               complete
    :return:
    """
    global __usserverId
    global __ossInfo

    id = hrandom.generate(5)

    # oss上传采取根目录加文件的形式
    fileName = os.path.basename(localPath)
    ossPath = id + os.path.splitext(localPath)[1]

    # 构造上传成功回调
    def ossUploadSuccess(ossData):
        data = {
            'localPath': localPath,
            'assetsPath': assetsPath,
            'ossPath': ossPath,
            'fileName': fileName
        }

        def success(data):
            status = data.get('status')
            if status is not None and status == 'fail':
                options.get('fail') and options.get('fail')()
            else:
                options.get('success') and options.get('success')(data)

        command.send(__usserverId, CommandEnum.OSS_FILE_UPLOAD_TO_ASSETS, data, {
            'success': success,
            'fail': options.get('fail'),
            'complete': options.get('complete')
        })

    oss.upload(localPath, ossPath, {
        'endpoint': __ossInfo.get('endpoint'),
        'bucketName': __ossInfo.get('bucketName'),
        'success': ossUploadSuccess,
        'fail': options.get('fail'),
        'complete': options.get('complete')
    })


def uploadToGcs(localPath, path, options={}):
    """
    上传本地文件到GCS
    :param localPath:      本地文件路径
    :param path:           数据存储路径
    :param options:
               bucketName  存储桶名称
               success
                   data
               fail
               complete
    :return:
    """
    global __usserverId
    global __ossInfo

    id = hrandom.generate(5)
    fileName = os.path.basename(localPath)
    ossPath = id + os.path.splitext(localPath)[1]

    def ossUploadSuccess(ossData):
        data = {
            'localPath':localPath,
            'path': path,
            'fileName': fileName,
            'ossPath': ossPath
        }

        def success(data):
            status = data.get('status')
            if status is not None and status == 'fail':
                options.get('fail') and options.get('fail')()
            else:
                options.get('success') and options.get('success')(data)

        command.send(__usserverId, CommandEnum.OSS_FILE_UPLOAD_TO_GCS, data, {
            'success': success,
            'fail': options.get('fail'),
            'complete':options.get('complete')
        })

    oss.upload(localPath, ossPath, {
        'endpoint': __ossInfo.get('endpoint'),
        'bucketName': __ossInfo.get('bucketName'),
        'success': ossUploadSuccess,
        'fail': options.get('fail'),
        'complete': options.get('complete')
    })


def downloadFromAssets(assetsPath, localPath, options={}):
    """
    从GEE资产管理下载数据到本地
    :param assetsPath:      资产路径
    :param localPath:       本地文件路径
    :param options:
               success
               fail
               complete
    :return:
    """
    global __usserverId
    global __ossInfo

    # oss上传采取根目录加文件的形式
    id = hrandom.generate(5)
    fileName = os.path.basename(localPath)
    ossPath = id + os.path.splitext(localPath)[1]

    def downloadSuccess(data):
        desPath = data.get('ossPath')

        def ossDownloadSuccess(ossData):
            oss.delete(ossData.get('path'), {
                'endpoint': __ossInfo.get('endpoint'),
                'bucketName': __ossInfo.get('bucketName')
            })
            options.get('success') and options.get('success')()

        oss.download(desPath, localPath, {
            'endpoint': __ossInfo.get('endpoint'),
            'bucketName': __ossInfo.get('bucketName'),
            'success': ossDownloadSuccess,
            'fail': options.get('fail'),
            'complete': options.get('complete')
        })

    data = {
        'assetsPath': assetsPath,
        'localPath': localPath,
        'ossPath': ossPath,
        'fileName':fileName
    }

    command.send(__usserverId, CommandEnum.OSS_FILE_DOWNLOAD_FROM_ASSETS, data, {
        'success': downloadSuccess,
        'fail': options.get('fail'),
        'complete': options.get('complete')
    })


def downloadFromGcs(path, localPath, options={}):
    """
    从GCS下载数据到本地
    :param path:            数据存储路径
    :param localPath:       本地文件路径
    :param options:
               bucketName
               success
                   localPath
               fail
               complete
    :return:
    """
    global __usserverId
    global __ossInfo

    id = hrandom.generate(5)
    fileName = os.path.basename(localPath)
    ossPath = id + os.path.splitext(localPath)[1]

    def downloadSuccess(ossData):
        desPath = data.get('ossPath')

        def ossDownloadSuccess(ossData):
            oss.delete(ossData.get('path'), {
                'endpoint': __ossInfo.get('endpoint'),
                'bucketName': __ossInfo.get('bucketName')
            })
            options.get('success') and options.get('success')({
                'path': path
            })

        oss.download(desPath, localPath, {
            'endpoint': __ossInfo.get('endpoint'),
            'bucketName': __ossInfo.get('bucketName'),
            'success': ossDownloadSuccess,
            'fail': options.get('fail'),
            'complete': options.get('complete')
        })

    data = {
        'path': path,
        'localPath': localPath,
        'ossPath': ossPath,
        'fileName': fileName
    }

    command.send(__usserverId, CommandEnum.OSS_FILE_DOWNLOAD_FROM_GCS, data, {
        'success': downloadSuccess,
        'fail': options.get('fail'),
        'complete': options.get('complete')
    })


def downloadFromDatasets(datasetsPath, localDir, conditions, options={}):
    """
    从GEE数据集中下载数据到本地
    当有多个Imgae下载时，需要拆分date，分多次command.send，不然会有发送回调的问题
    :param datasetsPath:       数据集路径 COPERNICUS/S2_SR
    :param localDir:           本地目录
    :param conditions
               lng:            经度   87.26
               lat:            纬度   44.19
               startDate:      起始日期  2020-06-01
               endDate:        结束日期  2020-06-05
               MGRS_TILE       瓦片代码
               clip:           裁剪框路径 assets中已存在
               isCalcNDVI:     True-计算NDVI，False-默认波段
               isCalcNDWI      是否计算NDWI, True or False
               isChlre         是否计算叶绿素，不能与isCalcNDVI同时选中，且bands需要包含B05和B07
               isNIR           是否计算近红外
               bands           波段筛选
               cloudValue:     云量值，指定不超过多少云量的数据进行查询
               nodataValue:    不可用区域筛选 NODATA_PIXEL_PERCENTAGE
               scale:          设置分辨率
    :param options:
               success
                   localPath  本地路径
               fail
                   message     消息
               complete
    :return:
    """
    global __usserverId
    global __ossInfo

    def oneImageSuccess(data):
        logger.info("执行回调函数oneImageSuccess")
        message = data.get('message')
        if message is not None:
            logger.info(message)
            options.get('fail') and options.get('fail')(message)
        else:
            dataDicList = []
            ossPath = data.get('path')
            if ossPath is not None:
                fileName = data.get('fileName')
                dataDicList.append({
                    'path': ossPath,
                    'fileName': fileName,
                    'dataName': data.get('name')
                })
            else:
                dataDicList = data.get('dataDicList')

            if dataDicList is not None:
                logger.info("开始执行下载,dataDicList={}".format(dataDicList))
                size = len(dataDicList)
                data['count'] = 0
                for dataDic in dataDicList:
                    fileNameCur = dataDic.get('fileName')
                    ossCurPath = dataDic.get('path')
                    localPath = None
                    if localDir.endswith('\\'):
                        localPath = localDir + fileNameCur
                    else:
                        localPath = localDir + "\\" + fileNameCur
                    dataDic['localPath'] = localPath
                    if conditions.get('old') is True:
                        data['localPath'] = localPath

                    def downloadSuccess(ossData):
                        data['count'] = data['count'] + 1
                        path = ossData.get('path')
                        logger.info("开始从oss下载到本地，path={}".format(ossData.get('localPath')))
                        oss.delete(path, {
                            'endpoint': __ossInfo.get('endpoint'),
                            'bucketName': __ossInfo.get('bucketName')
                        })

                        oldFlag = conditions.get('old')
                        if oldFlag is True:
                            options.get('success') and options.get('success')(data)
                        else:
                            if data['count'] == size:
                                logger.info("当前下载个数与data匹配，size={},")
                                logger.info("开始执行回调函数，data={}".format(data))
                                options.get('success') and options.get('success')(data)
                            else:
                                logger.info("当前下载个数size={},小于data['count']".format(size))

                    logger.info("oss开始下载，localPath={}".format(localPath))
                    oss.download(ossCurPath, localPath, {
                        'endpoint': __ossInfo.get('endpoint'),
                        'bucketName': __ossInfo.get('bucketName'),
                        'success': downloadSuccess,
                        'fail': options.get('fail'),
                        'complete': options.get('complete')
                    })

    # TODO: 怎么查询经纬度的时间列表支持，或者根据经纬度查找所在区域的代码，依次发送命令
    def queryFail():
        logger.info("获取当前最近可用时间失败，code={},startDate={},endDate={}".format(conditions.get('MGRS_TILE'), conditions.get('startDate'), conditions.get('endDate')))

    def querySuccess(dicDate):
        dateList = dicDate.get('date')
        size = len(dateList)
        if size > 0:
            for dateStr in dateList:
                dateRecent = dateStr[0:4] + '-' + dateStr[4:6] + '-' + dateStr[6:8]
                dateRecentTime = datetime.datetime.strptime(dateRecent, '%Y-%m-%d')
                endDateRecentTime = dateRecentTime + datetime.timedelta(1)
                endDateRecent = datetime.datetime.strftime(endDateRecentTime, '%Y-%m-%d')

                # TODO：克隆一个，然后修改startDate和endDate的值
                conditionsNew = conditions.copy()
                conditionsNew['startDate'] = dateRecent
                conditionsNew['endDate'] = endDateRecent

                command.send(id=__usserverId, name=CommandEnum.OSS_FILE_DOWNLOAD_FROM_DATASETS, data={
                    'datasetsPath': datasetsPath,
                    'conditions': conditionsNew
                }, options={
                    'success': oneImageSuccess,
                    'fail': options.get('fail'),
                    'complete': options.get('complete')
                })

    queryRecentDate(conditions=conditions, datasetsPath=datasetsPath,
                    options={
                        'success': querySuccess,
                        'fail': queryFail
                    })


def downloadImageFile(localDir, fileName, options={}):
    """
    下载影像文件
    :param localDir:     本地目录
    :param fileName:     文件名称
    :param options:
               success
                   ossData
                       path
                       localPath
               fail
               complete
    :return:
    """
    global __usserverId
    global __ossInfo

    def downloadSuccess(ossData):
        path = ossData.get('path')
        logger.info("从oss下载到本地成功，path={}".format(ossData.get('localPath')))
        oss.delete(path, {
            'endpoint': __ossInfo.get('endpoint'),
            'bucketName': __ossInfo.get('bucketName')
        })
        options.get('success') and options.get('success')(ossData)

    def success(path):
        localPath = os.path.join(localDir, fileName)
        oss.download(path, localPath, {
            'endpoint': __ossInfo.get('endpoint'),
            'bucketName': __ossInfo.get('bucketName'),
            'success': downloadSuccess,
            'fail': options.get('fail'),
            'complete': options.get('complete')
        })

    command.send(id=__usserverId, name=CommandEnum.OSS_FILE_DOWNLOAD_IMAGE_FILE, data={
        'fileName': fileName
    }, options={
        'success': success,
        'fail': options.get('fail'),
        'complete': options.get('complete')
    })


def queryRecentDate(conditions, datasetsPath='COPERNICUS/S2_SR', options={}):
    """
    查询最近数据可用时间
    :param conditions:
               startDate      开始时间
               endDate        结束时间
               MGRS_TILE      瓦片代码
               cloudValue:    云量值，指定不超过多少云量的数据进行查询
               nodataValue:   不可用区域筛选 NODATA_PIXEL_PERCENTAGE
               isOne:         是否当前最近的一个日期
    :param datasetsPath:      数据集路径
    :param options:
               success
                   date       日期
                   code       代码
               fail
               complete
    :return:
    """
    global __usserverId
    command.send(id=__usserverId, name=CommandEnum.RECENT_QUERY_DATE, data={
        'datasetsPath': datasetsPath,
        'conditions': conditions
    }, options={
        'success': options.get('success'),
        'fail': options.get('fail'),
        'complete': options.get('complete')
    })


def queryRecentAvalilable(conditions, datasetsPath='COPERNICUS/S2_SR'):
    """
    查询最近一幅图像的时间，格式20200805
    :param datasetsPath:      数据集路径
    :param conditions:
               startDate      开始时间
               endDate        结束时间
               MGRS_TILE      瓦片代码
               cloudValue:    云量值，指定不超过多少云量的数据进行查询
               nodataValue:   不可用区域筛选 NODATA_PIXEL_PERCENTAGE
               isOne:         是否当前最近的一个日期
    :return:
    """
    result = None
    if conditions is None:
        return result

    startDate = conditions.get('startDate')
    endDate = conditions.get('endDate')
    tile = conditions.get('MGRS_TILE')
    if startDate is not None and endDate is not None and tile is not None:
        imageColl = ee.ImageCollection(datasetsPath)
        imageColl = imageColl.filterDate(startDate, endDate)
        if imageColl is not None:
            imageColl = imageColl.filter(ee.Filter.eq('MGRS_TILE', tile))
            if imageColl is not None:
                imageColl = imageColl.sort('system:index', False)
                if imageColl is not None:
                    if imageColl.size() is not None:
                        size = imageColl.size().getInfo()
                        if size > 0:
                            dateList = []
                            for i in range(size):
                                image = imageColl.toList(imageColl.size()).get(i)
                                image = ee.Image(image)

                                cloudValue = conditions.get("cloudValue")
                                if cloudValue is not None:
                                    cloud = image.get("CLOUDY_PIXEL_PERCENTAGE")
                                    if cloud is not None:
                                        cloudyPixel = cloud.getInfo()
                                        if cloudyPixel > cloudValue:
                                            continue
                                nodataValue = conditions.get("nodataValue")
                                if nodataValue is not None:
                                    nodata = image.get("NODATA_PIXEL_PERCENTAGE")
                                    if nodata is not None:
                                        nodataPixel = nodata.getInfo()
                                        if nodataPixel > nodataValue:
                                            continue
                                name = __parseName(image)
                                nameCur = name[0:8]
                                if conditions.get('isOne') is True:
                                    result = nameCur
                                    break

                                # 影像会出现重复的日期，暂时去掉
                                if dateList.__contains__(nameCur) is False:
                                    dateList.append(nameCur)
                            if len(dateList) > 0:
                                result = dateList
    return result


def queryModisDate(conditions, datasetsPath='MODIS/006/MOD11A1'):
    result = None
    if conditions is None:
        return result

    startDate = conditions.get('startDate')
    endDate = conditions.get('endDate')
    if startDate is not None and endDate is not None:
        imageColl = ee.ImageCollection(datasetsPath)
        imageColl = imageColl.filterDate(startDate, endDate)
        if imageColl is not None:
            imageColl = imageColl.sort('system:index', False)
            if imageColl is not None:
                if imageColl.size() is not None:
                    size = 0
                    try:
                        size = imageColl.size().getInfo()
                    except:
                        logger.info("获取影像size失败")

                    if size > 0:
                        dateList = []
                        for i in range(size):
                            image = imageColl.toList(imageColl.size()).get(i)
                            image = ee.Image(image)
                            id = image.id().getInfo()
                            names = id.split('_')
                            dateCur = names[0] + names[1] + names[2]
                            if dateList.__contains__(dateCur) is False:
                                dateList.append(dateCur)
                        result = dateList
    return result


def ossFileDownloadFromAssets(assetsPath, localPath, bucketName, options={}):
    """
    从Earth Engine Assets上下载数据到本地目录
    1. assetsPath获取到数据ImageCollection
    2. 导出到Google Cloud
    3. Google Cloud下载到US Server本地
    4. US Server本地FTP传输到localPath
    5. 成功回调，删除US Server本地数据，GCS数据
    :param assetsPath:         Assets数据路径
    :param localPath:         本地文件路径
    :param bucketName:        存储桶名称
    :param options:
               success
               fail
               complete
    :return:
    """
    filename = os.path.basename(localPath)
    path = '\\' + filename

    # 构造上传GCS成功回调函数
    def gee2gcsSuccess(bucketPath):
        __downloadGCS2Temppath(bucketName, path, localPath, isDelGcs=True, options=options)

    optionsGcs = {
        'success': gee2gcsSuccess,
        'fail': options.get('fail'),
        'complete': options.get('complete')
    }

    logger.info("开始由assets传输至gcs")
    gcs.gee2gcs(assetsPath, bucketName, path.split('.')[0], optionsGcs)


def ossFileUploadToAssets(localPath, assetsPath, bucketName, options={}):
    """
    上传本地数据到Earth Engine Assets
    1. FTP传输localPath到US Server
    2. US Server上传到Google Cloud
    3. Google Cloud传输到Earth Engine Assets
    4. 成功回调，删除中间过程数据
    :param localPath:           本地路径
    :param assetsPath:          Assets数据路径
    :param bucketName:          存储桶名称
    :param options:
               success
               fail
               complete
    :return:
    """
    # 构建临时存储路径
    filename = os.path.basename(localPath)
    path = '\\' + filename

    def success(arg):
        __upload2GcsSuccess(localPath, bucketName, path, assetsPath, isDelGcs=True, options=options)

    logger.info("开始由本地数据上传到GCS")
    gcs.upload(localPath, bucketName, path, {
        'success': success,
        'fail': options.get('fail'),
        'complete': options.get('complete'),
    })


def ossFileDownloadFromGcs(bucketName, path, localPath, options={}):
    """
    从Google Cloud下载数据到本地
    1. 访问bucket的Path路径，获取数据
    2. 下载数据到US Server本地
    3. US Server本地传输到localPath
    4. 成功回调，删除US Server本地数据
    :param bucketName:      存储桶名称
    :param path:            存储路径
    :param localPath:       本地路径
    :param options:
               success
               fail
               complete
    :return:
    """
    __downloadGCS2Temppath(bucketName, path, localPath, isDelGcs=False, options=options)


def ossFileUploadToGcs(localPath, bucketName, path, options={}):
    """
    服务器上传到Google Cloud
    1. FTP传输localPath到US Server
    2. US Server上传到Google Cloud
    3. 成功回调，删除中间过程数据
    :param localPath:    本地文件路径
    :param bucketName:   存储桶名称
    :param path:         存储路径
    :param options:
               success
               fail
               complete
    :return:
    """
    gcs.upload(localPath, bucketName, path, options)


def ossFileDownloadFromDatasets(datasetsPath, conditions, localDir, bucketName, options={}):
    """
    从Earth Engine 数据集路径下载数据
    1. datasetsPath获取到数据ImageColection
    2. 导出到Google Cloud
    3. Google Cloud下载到US Server本地
    4. US Server本地FTP传输到localPath
    5. 成功回调，删除US Server本地数据，GCS数据
    :param datasetsPath:      数据集路径
    :param localDir:          本地路径目录
    :param conditions         查询条件
               lng:           经度 113.2
               lat:           纬度 30.28
               startDate:    开始日期 2020-03-01
               endDate:      结束日期 2020-03-20
               MGRS_TILE     瓦片代码
               cloudValue    云量值，指定不超过多少云量的数据进行查询
               clip          裁剪路径，assetsPath
               isCalcNDVI    是否计算NDVI, True or False
               isCalcNDWI    是否计算NDWI, True or False
               isChlre       是否计算叶绿素，不能与isCalcNDVI同时选中，且bands需要包含B05和B07
               isNIR         是否计算近红外
               bands         波段筛选
               nodataValue:  不可用区域筛选 NODATA_PIXEL_PERCENTAGE
               scale：       设置分辨率
    :param options:
               success
               fail
               complete
    :return:
    """
    global __ossInfo
    point = None
    date = None
    if conditions.get('lng') is not None and conditions.get('lat') is not None:
        point = str(conditions.get('lng')) + ',' + str(conditions.get('lat'))
    if conditions.get('startDate') is not None and conditions.get('endDate') is not None:
        date = conditions.get('startDate') + ',' + conditions.get('endDate')

    imageColl = __queryData(datasetsPath, point, date, conditions.get('MGRS_TILE'), conditions.get('coords'))
    if imageColl is not None:
        if imageColl.size() is not None:
            size = 0
            try:
                size = imageColl.size().getInfo()
            except:
                logger.info("查询完get size失败")

            # 如果是MODIS数据，需要median
            if datasetsPath == DatasetsPathType.MODIS_MOD11A1 or datasetsPath == DatasetsPathType.MODIS_MOD11A2:
                if size > 0:
                    coords = conditions.get('coords')
                    if coords is None:
                        coords = [135, 53.52, 73, 15.6]

                    sysID = imageColl.first().get('system:id').getInfo()
                    bands16 = ['LST_Day_1km', 'LST_Night_1km', 'Clear_day_cov', 'Clear_night_cov']
                    bands8 = ['QC_Day', 'Day_view_time', 'Day_view_angle', 'QC_Night', 'Night_view_time', 'Night_view_angle', 'Emis_31', 'Emis_32']
                    image16 = imageColl.select(bands16).mosaic()
                    image8 = imageColl.select(bands8).mosaic()
                    image16 = image16.clip(ee.Geometry.Rectangle(coords))
                    image8 = image8.clip(ee.Geometry.Rectangle(coords))
                    names = sysID.split('/')

                    imgName = names[-2] + '_' + names[-1]
                    gcsPath16 = imgName + '_UINT16.tif'
                    gcsPath8 = imgName + '_UINT8.tif'

                    dataList = []
                    def imgSuccess(desLocalPath):
                        id = hrandom.generate(5)
                        ossPath = id + os.path.splitext(desLocalPath)[1]

                        def ossSuccess(ossData):
                            path = ossData.get('path')
                            localPath = ossData.get('localPath')
                            os.remove(localPath)
                            dataList.append({
                                'path': path,
                                'fileName': os.path.basename(localPath),
                                'name': os.path.basename(localPath).split('.')[0]
                            })

                            if len(dataList) == 2:
                                gcsPathList = [gcsPath16, gcsPath8]
                                dataPathOne = dataList[0].get('fileName')
                                dataPathTwo = dataList[1].get('fileName')
                                if dataPathOne != dataPathTwo and dataPathOne in gcsPathList and dataPathTwo in gcsPathList:
                                    logger.info("地温数据上传成功，执行回调函数，localPath={}".format(localPath))
                                    options.get('success') and options.get('success')({
                                        'dataDicList': dataList
                                    })

                        oss.upload(desLocalPath, ossPath, {
                            'endpoint': __ossInfo.get('endpoint'),
                            'bucketName': __ossInfo.get('bucketName'),
                            'success': ossSuccess,
                            'fail': options.get('fail'),
                            'complete': options.get('complete')
                        })
                    __image2Gcs(image16, imgName+"_UINT16", bucketName, gcsPath16, localDir, {
                        'scale': 1000,
                        'region': ee.Geometry.Rectangle(coords=coords),
                        'success': imgSuccess,
                        'fail': options.get('fail'),
                        'complete': options.get('complete')
                    })

                    __image2Gcs(image8, imgName+"_UINT8", bucketName, gcsPath8, localDir, {
                        'scale': 1000,
                        'region': ee.Geometry.Rectangle(coords=coords),
                        'success': imgSuccess,
                        'fail': options.get('fail'),
                        'complete': options.get('complete')
                    })
            else:
                if size == 0:
                    logger.info("当前查询到数据集影像个数为0")
                    options.get('success') and options.get('success')({
                        'message': "当前查询到数据集影像个数为0"
                    })
                    options.get('complete') and options.get('complete')()
                elif size > MAX_DOWNLOAD_IMAGE_COUNT:
                    logger.info("当前查询到数据集影像个数超过 {} 个， 请修改条件后下载".format(size))
                    options.get('success') and options.get('success')({
                        'message': "当前查询到数据集影像个数超过 {} 个， 请修改条件后下载".format(size)
                    })
                    options.get('complete') and options.get('complete')()
                elif 0 < size < MAX_DOWNLOAD_IMAGE_COUNT:
                    dataDicList = []
                    if datasetsPath == DatasetsPathType.SENTINEL_LEVEL_2A:
                        oldFlag = conditions.get('old')
                        if oldFlag is True:
                            for i in range(size):
                                image = ee.Image(imageColl.toList(imageColl.size()).get(i))
                                image = ee.Image(image)
                                cloud = image.get("CLOUDY_PIXEL_PERCENTAGE")
                                nodata = image.get("NODATA_PIXEL_PERCENTAGE")
                                productID = image.get("PRODUCT_ID")
                                code = image.get('MGRS_TILE').getInfo()
                                cloudyPixel = None
                                if cloud is not None:
                                    cloudyPixel = cloud.getInfo()

                                nodataPixel = None
                                if nodata is not None:
                                    nodataPixel = nodata.getInfo()

                                name = __parseName(image)
                                if conditions.get('isNIR') is True:
                                    nir = image.select('B8')
                                    nir = nir.expression("nir/16", {'nir': nir}).rename('NIR')
                                    nir = nir.uint8()
                                    nir = image.addBands(nir)
                                    image = nir.select(['NIR', 'TCI_R', 'TCI_G'])
                                    name = name + '_NIR'
                                elif conditions.get('isChlre') is True:
                                    name = name + '_CHRE'
                                    image = image.expression("1/(re1/re3)", {
                                        're1': image.select('B5'),
                                        're3': image.select('B7')
                                    })

                                if conditions.get('isCalcNDVI') is True:
                                    name = name + '_NDVI'
                                    image = image.normalizedDifference(["B8", "B4"])

                                if conditions.get('isCalcNDWI') is True:
                                    name = name + '_NDWI'
                                    image = image.normalizedDifference(["B3", "B8"])

                                cloudValue = conditions.get('cloudValue')
                                if cloudValue is not None and cloudyPixel is not None:
                                    if cloudyPixel > cloudValue:
                                        logger.info("cloud={}云量超过设定值，不做下载操作".format(cloudyPixel))
                                        return

                                nodataValue = conditions.get('nodataValue')
                                if nodataValue is not None and nodataPixel is not None:
                                    if nodataPixel > nodataValue:
                                        logger.info("nodata={}超过设定值，不做下载操作".format(nodataPixel))
                                        return

                                gcsPath = name + '.tif'
                                def imgSuccess(desLocalPath):
                                    id = hrandom.generate(5)
                                    fileName = os.path.basename(desLocalPath)
                                    ossPath = id + os.path.splitext(desLocalPath)[1]

                                    def ossSuccess(ossData):
                                        path = ossData.get('path')
                                        os.remove(ossData.get('localPath'))
                                        dataParam = {
                                            'path': path,
                                            'fileName': fileName,
                                            'name': name,
                                            'code': code,
                                            'productID': productID.getInfo(),
                                            'cloudyPixel': cloudyPixel,
                                            'nodata': nodataPixel,
                                        }
                                        logger.info("oss数据上传成功，开始执行回调函数")
                                        options.get('success') and options.get('success')(dataParam)

                                    oss.upload(desLocalPath, ossPath, {
                                        'endpoint': __ossInfo.get('endpoint'),
                                        'bucketName': __ossInfo.get('bucketName'),
                                        'success': ossSuccess,
                                        'fail': options.get('fail'),
                                        'complete': options.get('complete')
                                    })
                                __image2Gcs(image, name, bucketName, gcsPath, localDir, {
                                    'success': imgSuccess,
                                    'fail': options.get('fail'),
                                    'complete': options.get('complete')
                                })
                        else:
                            # 如果是一个Imgae，则直接下载即可，如果是同一个CODE和Date出现多幅，则需要融合
                            image = None
                            imageFirst = None
                            region = None
                            image10Scale = None
                            image10Int8Scale = None
                            image20Scale = None
                            image20Int8Scale = None
                            image60Scale = None
                            project = None
                            if size > 1:
                                image10Scale = 10
                                image10Int8Scale = 10
                                image20Scale = 20
                                image20Int8Scale = 20
                                image60Scale = 60

                                # 获取坐标系信息
                                project = imageColl.first().select('B2').projection()
                                imageFirst = ee.Image(imageColl.toList(imageColl.size()).get(0))

                                ringFirst = imageFirst.get('system:footprint').getInfo()
                                ringFirst = ee.Geometry(ringFirst)
                                for i in range(1, size):
                                    imageCur = imageColl.toList(imageColl.size()).get(i)
                                    imageCur = ee.Image(imageCur)
                                    ringCur = imageCur.get('system:footprint').getInfo()
                                    ringFirst = ringFirst.union(ringCur)
                                region = ringFirst.convexHull()
                                image = imageColl.mosaic()
                            elif size == 1:
                                image = imageColl.toList(imageColl.size()).get(0)
                                image = ee.Image(image)

                            # 获取影像元数据信息
                            imageMeta = None
                            if size > 1:
                                imageMeta = imageFirst
                            else:
                                imageMeta = image
                            properties = __getS2AProperties(imageMeta)
                            mgrsTile = imageMeta.get('MGRS_TILE')
                            name = __parseName(imageMeta)

                            # 设置10米，20米，60米分为3个tif下载
                            image10 = image.select(['B2', 'B3', 'B4', 'B8', 'AOT', 'WVP'])
                            image10Int8 = image.select(['TCI_R', 'TCI_G', 'TCI_B'])
                            image20 = image.select(['B5', 'B6', 'B7', 'B8A', 'B11', 'B12'])
                            image20Int8 = image.select(['SCL', 'MSK_CLDPRB', 'MSK_SNWPRB'])
                            image60 = image.select(['B1', 'B9', 'QA60'])

                            def success(desLocalPath):
                                id = hrandom.generate(5)
                                paths = os.path.splitext(desLocalPath)
                                ossPath = id + paths[0].split('\\')[-1] + paths[1]

                                def ossSuccess(ossData):
                                    path = ossData.get('path')
                                    localPath = ossData.get('localPath')
                                    fileName = os.path.basename(localPath)
                                    os.remove(localPath)
                                    dataDicList.append({'path': path, 'fileName': fileName, 'dataName': fileName.split('.')[0]})
                                    logger.info("当前fileName={},当前dataDicList={}".format(fileName, dataDicList))

                                    # TODO:判断是否有5个同一个name的字典选项，如果有，回调，并且删除这些记录
                                    dataNewList = isHasFiveFile(dataDicList, name)
                                    if dataNewList is not None:
                                        dataParam = {
                                            'dataDicList': dataNewList,
                                            'code': mgrsTile.getInfo(),
                                            'name': name,
                                            'properties': properties
                                        }
                                        logger.info("开始执行回调函数，name={}".format(name))
                                        options.get('success') and options.get('success')(dataParam)
                                        for dataNew in dataNewList:
                                            dataDicList.remove(dataNew)
                                    else:
                                        logger.info("当前数据不足5个，localPath={}".format(localPath))

                                oss.upload(desLocalPath, ossPath, {
                                    'endpoint': __ossInfo.get('endpoint'),
                                    'bucketName': __ossInfo.get('bucketName'),
                                    'success': ossSuccess,
                                    'fail': options.get('fail'),
                                    'complete': options.get('complete')
                                })

                            logger.info("开始传输影像到GCS")
                            gcsPath10 = name + '_10_UINT16.tif'
                            gcsPath10Int8 = name + '_10_UINT8.tif'
                            gcsPath20 = name + '_20_UINT16.tif'
                            gcsPath20Int8 = name + '20_UINT8.tif'
                            gcsPath60 = name + '_60_UINT16.tif'

                            __image2Gcs(image10, name+'_10_UINT16', bucketName, gcsPath10, localDir, {
                                'scale': image10Scale,
                                'region': region,
                                'crs': project,
                                'description': name+'_10_UINT16',
                                'success': success,
                                'fail': options.get('fail'),
                                'complete': options.get('complete')
                            })
                            __image2Gcs(image10Int8, name+'_10_UINT8', bucketName, gcsPath10Int8, localDir, {
                                'scale': image10Int8Scale,
                                'region': region,
                                'crs': project,
                                'description': name+'_10_UINT8',
                                'success': success,
                                'fail': options.get('fail'),
                                'complete': options.get('complete')
                            })
                            __image2Gcs(image20, name+'_20_UINT16', bucketName, gcsPath20, localDir, {
                                'scale': image20Scale,
                                'region': region,
                                'crs': project,
                                'description': name+'_20_UINT16',
                                'success': success,
                                'fail': options.get('fail'),
                                'complete': options.get('complete')
                            })
                            __image2Gcs(image20Int8, name+'_20_UINT8', bucketName, gcsPath20Int8, localDir, {
                                'scale': image20Int8Scale,
                                'region': region,
                                'crs': project,
                                'description': name+'_20_UINT8',
                                'success': success,
                                'fail': options.get('fail'),
                                'complete': options.get('complete')
                            })
                            __image2Gcs(image60, name+'_60_UINT16', bucketName, gcsPath60, localDir, {
                                'scale': image60Scale,
                                'region': region,
                                'crs': project,
                                'description': name+'_60_UINT16',
                                'success': success,
                                'fail': options.get('fail'),
                                'complete': options.get('complete')
                            })
    else:
        logger.info("未查询到影像数据")


def isHasFiveFile(dataDicList, name):
    count = 0
    dataNewList = []
    for dataDic in dataDicList:
        fileName = dataDic.get('fileName')
        if fileName.startswith(name) is True:
            dataDicNew = {
                'path': dataDic.get('path'),
                'fileName': dataDic.get('fileName'),
                'dataName': dataDic.get('dataName')
            }
            dataNewList.append(dataDicNew)
            count = count + 1
    if count == 5:
        logger.info("isHasFiveFile获取name列表成功")
    else:
        logger.info("isHasFiveFile获取name列表失败")
        dataNewList = None
    return dataNewList


def ossFileDownloadImageFile(localDir, fileName, options={}):
    """
    下载影像文件并上传oss
    :param localDir:             本地目录
    :param fileName:             文件名称
    :param options:
               success
                   localPath
               fail
               complete
    :return:
    """
    global __ossInfo
    # 通过名称解析查询条件
    names = fileName.split('_')
    code = names[2][1:]
    pixel = names[3]
    dataType = os.path.splitext(names[4])[0]
    startDate = names[0][0:4] + '-' + names[0][4:6] + '-' + names[0][6:8]
    startDataTime = datetime.datetime.strptime(startDate, '%Y-%m-%d')
    endDateTime = startDataTime + datetime.timedelta(1)
    endDate = datetime.datetime.strftime(endDateTime, '%Y-%m-%d')

    def fileSuccess(localPath):
        ossPath = os.path.basename(localPath)

        def ossSuccess(ossData):
            logger.info("上传oss成功，path={}".format(ossData.get('path')))
            path = ossData.get('path')
            ossLocalPath = ossData.get('localPath')
            os.remove(ossLocalPath)
            options.get('success') and options.get('success')(path)

        oss.upload(localPath, ossPath, {
            'endpoint': __ossInfo.get('endpoint'),
            'bucketName': __ossInfo.get('bucketName'),
            'success': ossSuccess,
            'fail': options.get('fail'),
            'complete': options.get('complete')
        })

    def fileFail():
        options.get('fail') and options.get('fail')()

    ossDownloadImageFile(code, startDate, endDate, pixel,
                             dataType, localDir, fileName, options={
                                'success': fileSuccess,
                                'fail': fileFail
                            })


def ossDownloadImageFile(code, startDate, endDate, pixel,
                             dataType, localDir, fileName, options={}):
    """
    从GEE上下载数据到本地
    :param code:           区域代码
    :param startDate:      开始日期 yyyy-mm-dd
    :param endDate:        结束日期 yyyy-mm-dd
    :param pixel:          像素，整数
    :param dataType:       数据类型UINT16,UINT8等
    :param localDir:       本地目录
    :param fileName:       文件名称（带后缀）
    :param options:
               success
                   localPath
               fail
               complete
    :return:
    """
    if code is not None and startDate is not None and endDate is not None:
        coll = ee.ImageCollection(DatasetsPathType.SENTINEL_LEVEL_2A)
        coll = coll.filter(ee.Filter.eq('MGRS_TILE', code))
        coll = coll.filterDate(startDate, endDate)
        if coll is not None:
            if coll.size() is not None:
                size = 0
                try:
                    size = coll.size().getInfo()
                except:
                    logger.info("查询完get size失败")
                if size > 0:
                    name = None
                    image = None
                    region = None
                    project = None
                    imageScale = None
                    if size > 1:
                        imageScale = int(pixel)
                        project = coll.first().select('B2').projection()
                        imageFirst = ee.Image(coll.toList(coll.size()).get(0))

                        ringFirst = imageFirst.get('system:footprint').getInfo()
                        ringFirst = ee.Geometry(ringFirst)
                        for i in range(1, size):
                            imageCur = coll.toList(coll.size()).get(i)
                            imageCur = ee.Image(imageCur)
                            ringCur = imageCur.get('system:footprint').getInfo()
                            ringFirst = ringFirst.union(ringCur)
                        region = ringFirst.convexHull()
                        image = coll.mosaic()
                        name = __parseName(imageFirst)
                    elif size == 1:
                        image = coll.toList(coll.size()).get(0)
                        image = ee.Image(image)
                        name = __parseName(image)

                    # 根据分辨率和数据类型挑选波段，因为只有相同分辨率和数据类型才能在一个TIF文件中
                    if pixel == '10':
                        if dataType == 'UINT16':
                            image = image.select(['B2', 'B3', 'B4', 'B8', 'AOT', 'WVP'])
                        elif dataType == 'UINT8':
                            image = image.select(['TCI_R', 'TCI_G', 'TCI_B'])
                    elif pixel == '20':
                        if dataType == 'UINT16':
                            image = image.select(['B5', 'B6', 'B7', 'B8A', 'B11', 'B12'])
                        elif dataType == 'UINT8':
                            image = image.select(['SCL', 'MSK_CLDPRB', 'MSK_SNWPRB'])
                    elif pixel == '60':
                        image = image.select(['B1', 'B9', 'QA60'])

                    # TODO:数据目录目前写死，后期要可配置
                    dirName = 'rsdata'
                    newTask = Export.image.toDrive(image, description=fileName.split('.')[0], folder='rsdata',
                                                   fileNamePrefix=fileName.split('.')[0], maxPixels=1e13, crs=project,
                                                   region=region, scale=imageScale)

                    thread = threading.Thread(target=__downloadImageTask, args=(newTask, localDir, fileName, dirName, options))
                    thread.start()
                    logger.info("开始执行数据下载线程,fileName={}".format(fileName))
                else:
                    logger.info("未查询到当前区域和日期下的影像数据")


def __downloadImageTask(newTask, localDir, fileName, dirName, options={}):
    """
    执行数据下载任务
    :param newTask:      任务对象
    :param localDir:     本地目录
    :param fileName:     文件名称（带后缀）
    :param dirName:      目录名称
    :param options:
               success
                   localPath
               fail
               complete
    :return:
    """
    if newTask is not None:
        newTask.start()
        taskID = newTask.id

        flag = False
        timeNum = 0
        while True:
            # 超过3个小时，就跳出循环
            if timeNum > 10800:
                ee.data.cancelTask(taskID)
                logger.info("数据传输任务执行超时，取消任务")
                flag = False
                break

            taskStatuses = ee.data.getTaskStatus(taskID)
            if len(taskStatuses) > 0:
                status = taskStatuses[0].get('state')
                if status == 'COMPLETED':
                    flag = True
                    break
            time.sleep(20)
            timeNum = timeNum + 20
        if flag is True:
            drive.init()
            parentsID = drive.queryID(dirName)
            localPath = os.path.join(localDir, fileName)
            downloadFlag = drive.download(fileName, localPath, parentsID)
            if downloadFlag is True:
                logger.info("数据下载成功,fileName={}".format(fileName))
                deleteFlag = drive.delete(fileName)
                if deleteFlag is False:
                    logger.info("谷歌硬盘数据删除失败，fileName={}".format(fileName))
                options.get('success') and options.get('success')(localPath)
            else:
                options.get('fail') and options.get('fail')()
        else:
            logger.info("数据下载不成功,fileName={}".format(fileName))
            options.get('fail') and options.get('fail')()
    else:
        logger.info("任务为空，不执行")
        options.get('fail') and options.get('fail')()


def __downloadGCS2Temppath(bucket, path, localPath, isDelGcs=True, options={}):
    """
    下载GCS数据到指定路径
    :param bucket:            存储桶名称
    :param path:              存储路径
    :param localPath:         本地文件路径
    :param isDelGcs           是否删除GCS数据
    :param options:
               success
               fail
               complete
    :return:
    """
    # 如果本地文件存在，则加上时间后缀
    tempPath = __makeFileName(localPath)

    # 构造GCS下载到本地成功回调函数
    def success(desPath):
        if isDelGcs is True:
            gcs.delete(bucket, path)
        options.get('success') and options.get('success')(tempPath)

    logger.info("开始由GCS下载数据到本地")
    gcs.download(bucket=bucket, path=path, localPath=tempPath, options={
        'success': success,
        'fail': options.get('fail'),
        'complete': options.get('complete')
    })


def __makeFileName(filepath):
    """
    如果存在文件，则在文件名前面加上时间字符串，反之路径不变
    :param filepath:    文件全路径
    :return:            新的文件全路径
    """
    newFilePath = filepath
    if os.path.exists(filepath) is True:
        curTimeStr = time.strftime('%Y%m%d%H%M%S',time.localtime(time.time()))
        dir = os.path.dirname(filepath)
        filename = os.path.split(filepath)[-1]

        if dir.endswith('\\') is False:
            dir = dir + "\\"

        # 创建目录
        dir = dir + curTimeStr + "\\"
        os.mkdir(dir)

        newFilePath = dir + filename

    return newFilePath


def __selectS2ABands(image, bands=['B2', 'B3', 'B4', 'B8']):
    """
    筛选S2A波段，不然Export.image.toCloudStorage会有16和32的问题
    :param image:     需要筛选的影像
    :return:          筛选后的影像
    """
    result = image.select(bands)
    result = result.toInt16()
    return result


def __parseName(image):
    """
    解析影像名称
    :param image:  影像对象
    :return:       名称
    """
    name = image.get('system:id').getInfo()
    name = name.split('/')[-1]
    return name


def __upload2GcsSuccess(localPath, bucketName, path, assetsPath, isDelGcs=True, options={}):
    """
    上传GCS成功回调
    :param bucketName:      存储桶名称
    :param path:            存储路径
    :param assetsPath:      Assets路径
    :param isDelGcs:       是否需要删除GCS
    :param options:
               success
               fail
               complete
    :return:
    """
    def success(arg):
        if isDelGcs is True:
            gcs.delete(bucketName, path, {
                'fail': options.get('fail')
            })

        # 删除本地文件
        os.remove(localPath)
        options.get('success') and options.get('success')(arg)

    optionsNew = {
        'success': success,
        'fail': options.get('fail'),
        'complete': options.get('complete')
    }
    logger.info("开始由GCS数据传输到Assets目录")
    gcs.gcs2gee2(bucketName, path, assetsPath, optionsNew)


def __queryData(datasetsPath='COPERNICUS/S2_SR', point=None, date=None, code=None, coords=None):
    """
    1. 查询哨兵2数据, ['B1','B2','B3','B4','B5','B6','B7','B8','B8A','B9','B10','B11','B12']
    2. 目前只支持哨兵2的数据查询，后续会放开landsat等
    :param name:                 数据类型名称，目前只支持哨兵2
    :param point:                位置点，形式 '113.2,30.28'
    :param date:                 日期，形式 '2020-03-01,2020-03-20'
    :param code:                 区域代码，形式49RGQ
    :param options:
               success
               fail
               complete
    :return:   ImageCollection   影像集合
    """
    imageColl = ee.ImageCollection(datasetsPath)

    # 如果设置了点条件，查询点数据
    if point is not None:
        lonlat = point.split(',')
        if len(lonlat) == 2:
            lon = float(lonlat[0])
            lat = float(lonlat[1])
            lonlat = [lon, lat]
            pointGeo = ee.Geometry.Point(coords=lonlat)
            imageColl = imageColl.filterBounds(pointGeo)

    if datasetsPath == DatasetsPathType.MODIS_MOD11A1 or datasetsPath == DatasetsPathType.MODIS_MOD11A2:
        if coords is None:
            coords = [135, 53.52, 73, 15.6]
        imageColl = imageColl.filterBounds(ee.Geometry.Rectangle(coords=coords))

    # 如果设置了日期条件，日期筛选
    if date is not None:
        dates = date.split(',')
        if len(dates) == 2:
            startDate = dates[0]
            endDate = dates[1]
            imageColl = imageColl.filterDate(startDate, endDate)

    # 如果设置了代码条件，代码筛选
    if code is not None:
        imageColl = imageColl.filter(ee.Filter.eq('MGRS_TILE', code))

    return imageColl


def __image2Gcs(image, name, bucket, path, localDir, options={}):
    """
    影像上传GCS
    :param image:              影像对象
    :param name:               影像名称
    :param bucket:             存储桶名称
    :param path:               存储路径
    :param localDir:           本地目录
    :param options:
                success
                fail
                complete
    :return:
    """
    def success(desGcsPath):
        __image2GcsSuccess(name, bucket, path, localDir, options)

    logger.info("开始将影像传输到GCS")
    gcsPath = path.split('.')[0]

    gcs.image2gcs(image, bucket, gcsPath, {
        'scale': options.get('scale'),
        'region': options.get('region'),
        'crs': options.get('crs'),
        'description': options.get('description'),
        'success': success,
        'fail': options.get('fail'),
        'complete': options.get('complete')
    })


def __image2GcsSuccess(name, bucket, path, localDir, options={}):
    """
    影像下载到本地目录成功回调
    :param name:         影像名称
    :param bucket:       存储桶
    :param path:         存储路径
    :param options:
               success
               fail
               complete
    :return:
    """
    # 构造影像路径
    localPath = localDir + '\\' + name + '.tif'

    # 下载到本地目录
    __downloadGCS2Temppath(bucket, path, localPath, isDelGcs=True, options=options)


def __initMqtt(deviceid):
    """
    初始化MQTT信息
    :param deviceid:    设备id
    :return:
    """
    clientid = "o_{}".format(hrandom.generate(12))
    username = "tx2_o_n_{}_{}".format(deviceid, hrandom.generate(4))
    mqtt.init(clientid=clientid, username=username)
    command.init(deviceid)
    command.register(CommandEnum.RECEIVE_CMD, __receiveCommand)


def __receiveCommand(data, options={}):
    """
    收到美国服务器发送的回复命令
    :param data:
    :param options:
    :return:
    """
    if data is not None:
        resp = data.get('resp')
        if resp == 'OK':
            options.get('success') and options.get('success')({
                'status': 'OK'
            })


def __getUsserverFtpInfo():
    """
    获取美国服务器FTP信息
    :return:  FTP信息
    """
    ftpInfo = {
        'host': '47.252.81.121',
        'port': 6621,
        'username': 'hre',
        'password': 'H27Hgx*72oPql'
    }

    return ftpInfo


def getOssInfo():
    """
    获取oss连接信息
    :return:    oss连接信息
    """
    global __ossInfo
    ossInfo = {
        'endpoint': 'oss-accelerate.aliyuncs.com',
        'bucketName': 'h-usrs'
    }
    __ossInfo = ossInfo
    return ossInfo


def __getS2AProperties(image):
    result = None

    if image is not None:
        try:
            result = {
                "AOT_RETRIEVAL_ACCURACY": image.get("AOT_RETRIEVAL_ACCURACY").getInfo(),
                "CLOUDY_PIXEL_PERCENTAGE": image.get("CLOUDY_PIXEL_PERCENTAGE").getInfo(),
                "CLOUD_COVERAGE_ASSESSMENT": image.get("CLOUD_COVERAGE_ASSESSMENT").getInfo(),
                "CLOUDY_SHADOW_PERCENTAGE": image.get("CLOUDY_SHADOW_PERCENTAGE").getInfo(),
                "DARK_FEATURES_PERCENTAGE": image.get("DARK_FEATURES_PERCENTAGE").getInfo(),
                "DATASTRIP_ID": image.get("DATASTRIP_ID").getInfo(),
                "DATATAKE_IDENTIFIER": image.get("DATATAKE_IDENTIFIER").getInfo(),
                "DATATAKE_TYPE": image.get("DATATAKE_TYPE").getInfo(),
                "DEGRADED_MSI_DATA_PERCENTAGE": image.get("DEGRADED_MSI_DATA_PERCENTAGE").getInfo(),
                "FORMAT_CORRECTNESS": image.get("FORMAT_CORRECTNESS").getInfo(),
                "GENERAL_QUALITY": image.get("GENERAL_QUALITY").getInfo(),
                "GENERATION_TIME": image.get("GENERATION_TIME").getInfo(),
                "GEOMETRIC_QUALITY": image.get("GEOMETRIC_QUALITY").getInfo(),
                "GRANULE_ID": image.get("GRANULE_ID").getInfo(),
                "HIGH_PROBA_CLOUDS_PERCENTAGE": image.get("HIGH_PROBA_CLOUDS_PERCENTAGE").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B1": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B1").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B2": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B2").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B3": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B3").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B4": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B4").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B5": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B5").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B6": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B6").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B7": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B7").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B8": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B8").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B8A": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B8A").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B9": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B9").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B10": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B10").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B11": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B11").getInfo(),
                "MEAN_INCIDENCE_AZIMUTH_ANGLE_B12": image.get("MEAN_INCIDENCE_AZIMUTH_ANGLE_B12").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B1": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B1").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B2": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B2").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B3": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B3").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B4": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B4").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B5": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B5").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B6": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B6").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B7": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B7").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B8": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B8").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B8A": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B8A").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B9": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B9").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B10": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B10").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B11": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B11").getInfo(),
                "MEAN_INCIDENCE_ZENITH_ANGLE_B12": image.get("MEAN_INCIDENCE_ZENITH_ANGLE_B12").getInfo(),
                "MEAN_SOLAR_AZIMUTH_ANGLE": image.get("MEAN_SOLAR_AZIMUTH_ANGLE").getInfo(),
                "MEAN_SOLAR_ZENITH_ANGLE": image.get("MEAN_SOLAR_ZENITH_ANGLE").getInfo(),
                "MEDIUM_PROBA_CLOUDS_PERCENTAGE": image.get("MEDIUM_PROBA_CLOUDS_PERCENTAGE").getInfo(),
                "MGRS_TILE": image.get("MGRS_TILE").getInfo(),
                "NODATA_PIXEL_PERCENTAGE": image.get("NODATA_PIXEL_PERCENTAGE").getInfo(),
                "NOT_VEGETATED_PERCENTAGE": image.get("NOT_VEGETATED_PERCENTAGE").getInfo(),
                "PROCESSING_BASELINE": image.get("PROCESSING_BASELINE").getInfo(),
                "PRODUCT_ID": image.get("PRODUCT_ID").getInfo(),
                "RADIATIVE_TRANSFER_ACCURACY": image.get("RADIATIVE_TRANSFER_ACCURACY").getInfo(),
                "RADIOMETRIC_QUALITY": image.get("RADIOMETRIC_QUALITY").getInfo(),
                "REFLECTANCE_CONVERSION_CORRECTION": image.get("REFLECTANCE_CONVERSION_CORRECTION").getInfo(),
                "SATURATED_DEFECTIVE_PIXEL_PERCENTAGE": image.get("SATURATED_DEFECTIVE_PIXEL_PERCENTAGE").getInfo(),
                "SENSING_ORBIT_DIRECTION": image.get("SENSING_ORBIT_DIRECTION").getInfo(),
                "SENSING_ORBIT_NUMBER": image.get("SENSING_ORBIT_NUMBER").getInfo(),
                "SENSOR_QUALITY": image.get("SENSOR_QUALITY").getInfo(),
                "SOLAR_IRRADIANCE_B1": image.get("SOLAR_IRRADIANCE_B1").getInfo(),
                "SOLAR_IRRADIANCE_B2": image.get("SOLAR_IRRADIANCE_B2").getInfo(),
                "SOLAR_IRRADIANCE_B3": image.get("SOLAR_IRRADIANCE_B3").getInfo(),
                "SOLAR_IRRADIANCE_B4": image.get("SOLAR_IRRADIANCE_B4").getInfo(),
                "SOLAR_IRRADIANCE_B5": image.get("SOLAR_IRRADIANCE_B5").getInfo(),
                "SOLAR_IRRADIANCE_B6": image.get("SOLAR_IRRADIANCE_B6").getInfo(),
                "SOLAR_IRRADIANCE_B7": image.get("SOLAR_IRRADIANCE_B7").getInfo(),
                "SOLAR_IRRADIANCE_B8": image.get("SOLAR_IRRADIANCE_B8").getInfo(),
                "SOLAR_IRRADIANCE_B8A": image.get("SOLAR_IRRADIANCE_B8A").getInfo(),
                "SOLAR_IRRADIANCE_B9": image.get("SOLAR_IRRADIANCE_B9").getInfo(),
                "SOLAR_IRRADIANCE_B10": image.get("SOLAR_IRRADIANCE_B10").getInfo(),
                "SOLAR_IRRADIANCE_B11": image.get("SOLAR_IRRADIANCE_B11").getInfo(),
                "SOLAR_IRRADIANCE_B12": image.get("SOLAR_IRRADIANCE_B12").getInfo(),
                "SNOW_ICE_PERCENTAGE": image.get("SNOW_ICE_PERCENTAGE").getInfo(),
                "SPACECRAFT_NAME": image.get("SPACECRAFT_NAME").getInfo(),
                "THIN_CIRRUS_PERCENTAGE": image.get("THIN_CIRRUS_PERCENTAGE").getInfo(),
                "UNCLASSIFIED_PERCENTAGE": image.get("UNCLASSIFIED_PERCENTAGE").getInfo(),
                "VEGETATION_PERCENTAGE": image.get("VEGETATION_PERCENTAGE").getInfo(),
                "WATER_PERCENTAGE": image.get("WATER_PERCENTAGE").getInfo(),
                "WATER_VAPOUR_RETRIEVAL_ACCURACY": image.get("WATER_VAPOUR_RETRIEVAL_ACCURACY").getInfo()
            }
        except:
            logger.info("获取属性信息失败")

    return result


def addLogHanddle(logHandler):
    global logger
    logger.addHandler(logHandler)
