# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2020, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Levi
# @version  1.0.0
# @brief    geoserver数据发布
# @address  geoserver官网RESTful: https://docs.geoserver.org/stable/en/user/rest/index.html
#           GitHub:    https://github.com/GeoNode/geoserver-restconfig
#
#  History:
#      <author>     <time>     <version>   <desc>
#       Levi      2010-05-07     1.0.0   geoserver数据发布
#############################################################


import logging
import threading

from geoserver.catalog import Catalog
from geoserver.style import Style
from geoserver.util import shapefile_and_friends


__version__ = "1.0.0"


__all__ = [
    'init',
    'deinit',

    'getCatalog',

    'publishShp',
    'publishTif',

    'setStyle',
]


"""
# 常量
"""
GEOSERVER_REST_SUFFIX = "/geoserver/rest"


"""
# 枚举
"""


"""
# 全局变量
"""
# 目录对象集
__catalogs = [
    # {
    #     'id':  '',
    #     'catalog': None
    # }
]

# 日志对象
logger = logging.getLogger("geoserver")

# 打印级别
logging.basicConfig(level=logging.INFO)


def init(id, options={}):
    """
    初始化
    :param id:              唯一标识
    :param options:
               host         地址
               port         端口
               username     用户名
               password     密码
               success
               fail
               complete
    :return:
    """
    catalog = getCatalog(id)
    if catalog is None:
        url = __generateRESTUrl(options.get('host'), options.get('port'))
        if url is not None:
            catalog = Catalog(service_url=url, username=options.get('username'),
                              password=options.get('password'))
            __catalogs.append({
                'id': id,
                'catalog': catalog
            })
            logger.info('不存在该id的catalog，根据连接信息添加一个')

    if catalog is not None:
        options.get('success') and options.get('success')()
        options.get('complete') and options.get('complete')()


def deinit(id):
    """
    注销
    :param id:         唯一标识符
    :return:
    """
    catalog = getCatalog(id)
    if catalog is not None:
        __catalogs.remove(catalog)


def getCatalog(id):
    """
    根据id获取catalog
    :param id:         唯一标识符
    :return:           catalog对象
    """
    global __catalogs

    result = None
    for catalog in __catalogs:
        if catalog.get('id') == id:
            result = catalog.get('catalog')
            break

    return result


def publishTif(id, tifPath, layerName, options={}):
    """
    发布影像数据
    :param id:             唯一标识
    :param tifPath:        影像tif数据路径
    :param layerName:      发布数据名称，也是数据存储名称
    :param options:
               wsName      工作空间名称
               storeName   数据存储名称
               styleName   样式名称
               success
               fail
               complete
    :return:
    """
    options.setdefault('wsName', None)
    wsName = options.get('wsName')
    options.setdefault('styleName', None)
    styleName = options.get('styleName')

    storeName = options.get('storeName')
    if storeName is None:
        storeName = layerName

    catalog = getCatalog(id)
    if catalog is not None:
        # 工作空间设置，默认为空，取默认工作空间
        workspace = None
        if wsName is not None:
            workspace = catalog.get_workspace(wsName)

        # 开启发布线程
        thread = threading.Thread(target=__publishTifThread,
                                  args=(id, tifPath, catalog, workspace, storeName, layerName, styleName, options))
        thread.start()
        logger.info('开始发布影像数据:{}'.format(tifPath))


def publishShp(id, shpPath, layerName, options={}):
    """
    发布矢量shp数据
    :param id:              唯一标识
    :param shpPath:         shp矢量数据路径
    :param layerName:       图层名称，与数据存储名称相同
    :param options:
               wsName       工作空间名称
               styleName    样式名称
               success
               fail
               complete
    :return:
    """
    options.setdefault('wsName', None)
    wsName = options.get('wsName')
    options.setdefault('styleName', None)
    styleName = options.get('styleName')

    catalog = getCatalog(id)
    if catalog is not None:
        # 工作空间设置，默认为空，取默认工作空间
        workspace = None
        if wsName is not None:
            workspace = catalog.get_workspace(wsName)

        # 开启发布线程
        thread = threading.Thread(target=__publishShapeThread,
                                  args=(shpPath, catalog, workspace, layerName, styleName, options))
        thread.start()
        logger.info('开始发布影像数据:{}'.format(shpPath))


def setStyle(id, layerName, styleName, options={}):
    """
    设置图层样式
    :param id:             唯一标识
    :param layerName:      图层名称
    :param styleName:      样式名称
    :param options:
               wsName
               success
               fail
               complete
    :return:
    """
    catalog = getCatalog(id)
    if catalog is not None:
        layer = catalog.get_layer(layerName)
        if layer is not None:
            style = catalog.get_style(styleName)
            if style is not None:
                layer.default_style = style
                catalog.save(layer)
                options.get('success') and options.get('success')()
                options.get('complete') and options.get('complete')()
            else:
                logger.error("未找到样式 {}".format(styleName))
        else:
            logger.error("未找到图层 {}".format(layerName))
    else:
        logger.error("id:{} 获取catalog为空".format(id))


def __generateRESTUrl(host, port):
    """
    构造REST请求路径           e.g. http://localhost:8080/geoserver/rest
    如果host为域名，则port为空，e.g. http://map.tianxiao2.com/geoserver/rest
    :param host:         ip地址
    :param port:         端口
    :return:             rest路径
    """
    result = None
    if host is not None:
        if port is None:
            result = 'http://' + host + GEOSERVER_REST_SUFFIX
        else:
            result = 'http://' + host + ':' + str(port) + GEOSERVER_REST_SUFFIX
    return result


def __publishTifThread(id, tifPath, catalog, workspace, storeName, layerName, styleName, options={}):
    """
    发布影像数据线程
    :param tifPath:      影像路径
    :param storeName:    数据存储名称
    :param layerName:    发布数据名称
    :param workspace:    工作空间对象
    :param catalog:      管理目录对象
    :param options:
               success
               fail
               complete
    :return:
    """
    if catalog is not None:
        store = catalog.create_coveragestore(name=storeName, workspace=workspace,
                                             path=tifPath, create_layer=True,
                                             layer_name=layerName)
        if store is None:
            options.get('fail') and options.get('fail')()
        else:
            if styleName is not None:
                setStyle(id, layerName, styleName)
            options.get('success') and options.get('success')()
    options.get('complete') and options.get('complete')()


def __publishShapeThread(shpPath, catalog, workspace, layerName, styleName, options={}):
    """
    发布矢量数据线程
    :param data:
    :param catalog:
    :param workspace:
    :param layerName:
    :param styleName:
    :param options:
               success
               fail
               complete
    :return:
    """
    if shpPath.endswith('.shp'):
        shpDir = shpPath[:-4]
        shapefilePlusSidecars = shapefile_and_friends(shpDir)
        if shapefilePlusSidecars is not None:
            catalog.create_featurestore(name=layerName, data=shapefilePlusSidecars, workspace=workspace)
            if workspace is None:
                workspace = catalog.get_default_workspace()
            store = catalog.get_store(name=layerName, workspace=workspace)
            if store is not None:
                if styleName is not None:
                    layer = catalog.get_layer(layerName)
                    styleObj = Style(catalog=catalog, name=styleName)
                    layer.default_style = styleObj
                    catalog.save(layer)

                options.get('success') and options.get('success')()
                options.get('complete') and options.get('complete')()



