# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2020, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Levi
# @version  1.0.0
# @brief    drive磁盘数据管理，需要按照官网API提示生成credentials.json
#           运行程序后生成token.pickle，这两个文件作为环境配置文件拷贝
#           The file token.pickle stores the user's access and refresh tokens, and is
#           created automatically when the authorization flow completes for the first time.
# @address  官网：      https://drive.google.com/drive/my-drive
#           Referrence https://developers.google.com/drive/api/v3/reference
#           API网站：   https://developers.google.com/drive/api/v3/about-sdk
#
#  History:
#      <author>     <time>     <version>   <desc>
#       Levi     2021-01-26     1.0.0     磁盘数据管理
#############################################################
import io
import logging
import os
import pickle

from google.auth.transport.requests import Request
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.http import MediaIoBaseDownload, MediaFileUpload

__all__ = [
]


"""
# 常量
"""
# 默认授权磁盘增删改查权限，可以变更，如只读
# If modifying these scopes, delete the file token.pickle.
SCOPES = ['https://www.googleapis.com/auth/drive']


"""
# 枚举
"""


"""
# 全局变量
"""
logger = logging.getLogger(__name__)

# 资格证书
__credentials = None


def init(options={}):
    """
    初始化资格证书
    :param options:
               success
               fail
               complete
    :return:
    """
    global __credentials

    # 创建权限读取机制，如果存在就直接读取文件，不存在则创建Flow并保存文件
    __credentials = None
    picklePath = os.getcwd() + '\\' + 'token.pickle'
    if os.path.exists(picklePath):
        with open(picklePath, 'rb') as token:
            __credentials = pickle.load(token)
    else:
        logger.info("token.pickle文件不存在")
    if not __credentials or not __credentials.valid:
        if __credentials and __credentials.expired and __credentials.refresh_token:
            __credentials.refresh(Request())
        else:
            credentialsPath = os.getcwd() + '\\' + 'credentials.json'
            if os.path.exists(picklePath):
                flow = InstalledAppFlow.from_client_secrets_file(credentialsPath, SCOPES)
                if flow is not None:
                    __credentials = flow.run_local_server(port=0)
                else:
                    logger.info("创建flow失败")
            else:
                logger.info("credentials.json文件不存在")
        with open('token.pickle', 'wb') as token:
            pickle.dump(__credentials, token)
    if __credentials is not None:
        options.get('success') and options.get('success')()
    else:
        options.get('fail') and options.get('fail')()
    options.get('complete') and options.get('complete')()


def listDirFiles(dirName):
    """
    获取目录下的所有文件
    Search for files and folders  https://developers.google.com/drive/api/v3/search-files
    Search query terms            https://developers.google.com/drive/api/v3/ref-search-terms
    Files: list                   https://developers.google.com/drive/api/v3/reference/files/list
    :param dirName:    目录名称
    :return:           文件字典列表
    """
    global __credentials
    result = None

    if __credentials is not None:
        service = build('drive', 'v3', credentials=__credentials)
        dirID = queryID(dirName)
        if dirID is not None:
            results = service.files().list(q="'"+dirID+ "' in parents", fields="nextPageToken, files(id, name)").execute()
            result = results.get('files', [])
        else:
            logger.info("查询目录ID失败")
        service.close()
    return result


def uploadFile(path, name, parents=None):
    """
    上传文件
    :param path:        文件路径
    :param name:        文件名称
    :param parents:     父级目录
    :return:            True-成功  False-失败
    """
    global __credentials
    result = False

    if name is not None and path is not None:
        if __credentials is not None:
            service = build('drive', 'v3', credentials=__credentials)
            fileMetadata = {
                'name': name
            }
            if parents is not None:
                fileMetadata['parents'] = [parents]

            media = MediaFileUpload(path)
            file = service.files().create(body=fileMetadata, media_body=media, fields='id').execute()
            if file.get('id') is not None:
                result = True
            service.close()
    return result


def queryID(name):
    """
    根据目录名称查询目录或文件ID
    关于查询条件，参考https://developers.google.com/drive/api/v3/search-files
    :param name:      目录或文件名称
    :return:         目录或文件ID
    """
    global __credentials
    id = None
    if name is not None:
        if __credentials is not None:
            service = build('drive', 'v3', credentials=__credentials)
            qParam = "name='" + name + "'"
            dirResults = service.files().list(q=qParam, fields="nextPageToken, files(id, name)").execute()
            dirItems = dirResults.get('files', [])
            if len(dirItems) > 0:
                id = dirItems[0]['id']
            service.close()
    return id


def download(name, localPath, parents=None):
    """
    下载文件
    :param name:          文件名称
    :param localPath:     本地文件存储路径
    :param parents:       父级目录
    :return:              True-成功  False-失败
    """
    global __credentials
    result = False

    if name is not None and localPath is not None:
        if __credentials is not None:
            service = build('drive', 'v3', credentials=__credentials)
            try:
                id = queryID(name)
                if id is not None:
                    request = service.files().get_media(fileId=id)
                    fh = io.BytesIO()
                    downloader = MediaIoBaseDownload(fh, request)
                    done = False
                    # 下载过程中有可能报错，重试3次
                    try:
                        while done is False:
                            status, done = downloader.next_chunk()
                            logger.info("Download %d%%." % int(status.progress() * 100))
                    except:
                        try:
                            while done is False:
                                status, done = downloader.next_chunk()
                                logger.info("Download %d%%." % int(status.progress() * 100))
                        except:
                            while done is False:
                                status, done = downloader.next_chunk()
                            logger.info("Download %d%%." % int(status.progress() * 100))
                    logger.info("下载到内存完成，localPath={}".format(localPath))
                    with open(localPath, 'wb') as f:
                        f.write(fh.getvalue())
                        result = True
                    fh.close()
            except Exception as err:
                logger.error(err)
            service.close()
    return result


def delete(name):
    """
    从磁盘中删除文件
    :param name:    文件名称
    :return:        True-成功，False-失败
    """
    result = False
    global __credentials
    if name is not None:
        if __credentials is not None:
            id = queryID(name)
            if id is not None:
                try:
                    service = build('drive', 'v3', credentials=__credentials)
                    service.files().delete(fileId=id).execute()
                    service.close()
                    result = True
                except Exception as err:
                    result = False
                    logger.info("删除文件失败，name={},err={}".format(name, err))
            else:
                result = False
                logger.info("获取文件ID失败，name={}".format(name))
    return result


if __name__ == '__main__':
    os.environ['HTTP_PROXY'] = 'http://127.0.0.1:1080'
    os.environ['HTTPS_PROXY'] = 'http://127.0.0.1:1080'

    # init()

    # dirName = '20201110T025939_20201110T025938_T50RKU_60_UINT16.tif'
    # dirID = queryID(dirName)
    # print("第一个"+dirID)

    # name = '20201225T030131_20201225T030129_T50RKU_20_UINT8'
    # path = r'D:\wwwroot\geoserver_2.15.2\geoserver_data\_rs\20201225T030131_20201225T030129_T50RKU_20_UINT8.tif'
    # parents = '12TEV4yj2cV4ydpZ96aSQlhgKBnLTgQf-'
    # uploadFile(path, name, parents)

    # dirName = 'ImageDir'
    # fileDirs = listDirFiles(dirName)
    # for fileItem in fileDirs:
    #     id = fileItem['id']
    #     name = fileItem['name']
    #     print(id)
    #     print(name)

    # parents = '12TEV4yj2cV4ydpZ96aSQlhgKBnLTgQf-'
    # name = '20201225T030131_20201225T030129_T50RKU_20_UINT8.tif'
    # localPath = r'D:\UploadFiles\20201225T030131_20201225T030129_T50RKU_20_UINT8.tif'
    # download(name, localPath, parents)

    # name = '20201225T030131_20201225T030129_T50RKU_20_UINT8.tif'
    # result = delete(name)
    # print(result)

