# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2020, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Levi
# @version  1.0.0
# @brief    dubbo服务模块
#  History:
#      <author>     <time>     <version>   <desc>
#       Levi     2010-04-10     1.0.0      dubbo服务
#############################################################
import logging
import os
import configparser

from dubbo.client import DubboClient
from dubbo.server import DubboService
from dubbo.codec.hessian2 import new_object

import hrse.zookeeper as zookeeper


__version__ = "1.0.0"


__all__ = [
    'start',
    'stop',

    'JavaObject',

    'add',
    'invoke',
]


"""
# 常量
"""


"""
# 枚举
"""


"""
# 全局变量
"""
# Dubbo服务
__dubboService = None

# Dubbo服务列表
__dubboTempServices = {}

# Dubbo终端列表
__dubboClients = [
]

# 记录启动端口号
__dubboServicePort = None

# 重命名该方法
JavaObject = new_object

# 日志对象
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('dubbo')


def start(zookeeperConfig, dubboServiceConfig, options={}):
    """
    配置Dubbo服务
    1. 配置zookeeper
    2. 注册Dubbo服务
    :param zookeeperConfig:
                host
                port
    :param dubboServiceConfig:
                port
                app
                version
    :param options:
                success
                fail
                complete
    :return:
    """
    global __dubboService
    global __dubboServicePort
    global __dubboTempServices

    # 启动zookeeper
    zookeeper.start(zookeeperConfig.get("host"), zookeeperConfig.get("port"))
    logger.info("启动zookeeper中心成功")

    if __dubboService is None:
        __dubboService = DubboService(dubboServiceConfig.get("port"), dubboServiceConfig.get("app"))

    # 如果临时服务列表非空，则全部添加到服务中
    if __dubboTempServices is not None:
        for service, methods in __dubboTempServices.items():
            for method, handler in methods.items():
                __dubboService.add_method(service=service, method=method, handler=handler)

    #　TODO: 在ZooKeeper中会产生多个同名服务，今后需要实现删除已有服务的功能
    __dubboService.register(zookeeperConfig.get("host") + ':' + zookeeperConfig.get("port"), "")
    logger.info("注册服务到zookeeper中心成功")

    # 启动Dubbo服务
    __dubboService.start()
    logger.info("启动dubbo服务")

    __dubboServicePort = dubboServiceConfig.get("port")

    options.get("success")  and options.get("success")()
    options.get("complete") and options.get("complete")()


def stop(options={}):
    """
    停止Dubbo
    TODO:遍历__dubboTempServices，查找节点路径，删除对应节点
    1. 停止的时候需要删除register产生的服务节点，
    :param options:
                success
                fail
                complete
    :return:
    """
    global __dubboService
    global __dubboTempServices
    # 删除服务的节点
    if __dubboTempServices is not None:
        for dubboTempService in __dubboTempServices.keys():
            zookeeper.deleteDubboService(dubboTempService, __dubboServicePort)

    if __dubboService is not None:
        __dubboService.stop()
        __dubboService = None
        logger.info("停止dubbo服务")

    zookeeper.stop()
    logger.info("停止zookeeper中心服务")


def add(serviceName, methodName, handler, options={}):
    """
    添加服务的方法
    如果添加相同的服务和方法，则会覆盖旧的handler
    TODO:java后台的serviceVersion全部为空，故此处也应该为空
    1. 给该Service添加方法
    :param serviceName:
    :param methodName:
    :param handler:
    :param options:
                success
                fail
                complete
    :return:
    """
    global __dubboTempServices
    global __dubboService
    service_map = __dubboTempServices.setdefault(serviceName, {})
    service_map[methodName] = handler

    if __dubboService is not None:
        __dubboService.add_method(service=serviceName, method=methodName, handler=handler)

    options.get("success") and options.get("success")()
    options.get("complete") and options.get("complete")()


def invoke(serviceName, methodName, args, options={}, serviceVersion=''):
    """
    执行服务方法
    TODO: 还需要维护client的有效性；也就是说，如果节点失效，需要重新获取
    1. 先判断__dubboClients[]列表中是否已经存在该服务的client，如果存在，直接使用
    2. 从Zookeeper中心中获取该服务的地址；
    3. 建立client到目标地址
    :param serviceName: 服务名
    :param methodName: 方法名
    :param args: 执行方法参数
    :param options: 执行参数
               hasResponse:  是否有响应
               hasConvertip: 是否需要转换ip
               success:
               fail
               complete
    :param serviceVersion: 服务的版本
    :return:
    """
    global __dubboClients
    result = None

    services = zookeeper.getDubboServices(serviceName=serviceName)
    if services is None or len(services) == 0:
        logger.info("未获取到该服务名对应的服务")
        options.get("fail") and options.get("fail")()
        options.get("complete") and options.get("complete")()
    else:
        # TODO:解析path得到ip和port, 如果有多个，默认先执行第一个，后续再优化
        # 暂时未写如何获取每个服务权重，及决策调用哪个服务的机制
        infos = services.get("paths")[0].split(':')
        ip = infos[0]
        port = int(infos[1])

        # TODO：当所调用的服务重启时，该client会失效，会导致调用失败
        # 暂时屏蔽词代码，每次都new一个，可以解决问题，后续再优化
        # dubboClient = __getDubboClient(ip, port)
        # if dubboClient is None:
        #     logger.info("未获取到dubbo客户端，添加一个")
        #     dubboClient = DubboClient(ip, port)
        #     __dubboClients.append({
        #         'ip': ip,
        #         'port': port,
        #         'client': dubboClient,
        #     })

        if options.get("hasResponse") is True:
            result = DubboClient(ip, port).send_request_and_return_response(
                service_name=serviceName,
                service_version=serviceVersion,   # version必须为空
                method_name=methodName,
                args=args
            )
        else:
            DubboClient(ip, port).send_request_without_response(
                service_name=serviceName,
                service_version=serviceVersion,   # version必须为空
                method_name=methodName,
                args=args
            )

        logger.info("请求执行完毕")
        options.get("success") and options.get("success")()
        options.get("complete") and options.get("complete")()

    return result


def __getDubboClient(ip, port):
    """
    获得Dubbo连接终端
    :param ip:
    :param port:
    :return:
    """
    global __dubboClients

    for dubboClient in __dubboClients:
        if dubboClient.get("ip") == ip and dubboClient.get("port") == port:
            return dubboClient.get("client")

    return None


def __getConvertIp(ip):
    """
    根据旧ip映射到新ip
    :param ip: 旧ip
    :return:   新ip
    """
    result = None
    cf = configparser.ConfigParser()

    configPath = os.getcwd() + '\\hre.config'
    if os.path.exists(configPath) is True:
        cf.read(configPath)
        result = cf.get('ipconvert', ip)

    return result