# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2020, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Levi
# @version  1.0.0
# @brief    ftp数据传输  官网：  https://docs.python.org/3.7/library/ftplib.html
#                       源代码: https://github.com/python/cpython/blob/3.7/Lib/ftplib.py
#  History:
#      <author>     <time>     <version>   <desc>
#       Levi     2010-04-27     1.0.0      ftp数据传输
#############################################################
import threading
import time

__version__ = "1.0.0"


import os
import logging

from ftplib import FTP


__all__ = [
    'init',
    'deinit',

    'download',
    'upload',
]


"""
# 常量
"""


"""
# 枚举
"""


"""
# 全局变量
"""
# ftp客户端数组
__ftpClients = []

# 日志管理对象
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('ftp')


def init(id, options={}):
    """
    初始化
    :param id:              唯一标识
    :param options:
               host         地址
               port         端口
               username     用户名
               password     密码
               success
               fail
               complete
    :return:
    """
    ftpClient = __getFtp(id)
    if ftpClient is None:
        logger.info("未获取到ftp客户端，添加一个")
        ftpClient = FTP()
        __ftpClients.append({
            'id': id,
            'client': ftpClient,
        })

    # 连接FTP服务器
    ftpClient.connect(options.get('host'), options.get('port'))
    logger.info("连接成功")

    # 连接的用户名，密码如果匿名登录则用空串代替即可
    ftpClient.login(user=options.get('username'), passwd=options.get('password'))
    logger.info("登录成功")


def deinit(id):
    """
    退出
    :param id:  唯一标识
    :return:
    """
    for ftpClient in __ftpClients:
        ftpClient.get('id')
        ftp = __getFtp(id)
        if ftp is not None:
            ftp.quit()
    __ftpClients.clear()
    logger.info("退出成功")


def download(id, remotePath, localPath, options={}):
    """
    从远程目录下载数据
    :param id:           客户端唯一标识
    :param remotePath:   远程目录
    :param localPath:    本地路径
    :param options:
               success
               fail
               complete
    :return:
    """
    ftp = __getFtp(id)
    if ftp is not None:
        thread = threading.Thread(target=__download, args=(ftp, remotePath, localPath, options))
        thread.start()
        logger.info("启动FTP下载线程")


def upload(id, localPath, remotePath, options={}):
    """
    把本地文件上载到远程目录
    :param remotepath:     远程目录
    :param localpath:      本地路径
    :param options:
               success
               fail
               complete
    :return:
    """
    ftp = __getFtp(id)
    if ftp and ftp.sock is not None:
        thread = threading.Thread(target=__upload, args=(ftp, localPath, remotePath, options))
        thread.start()
        logger.info("启动FTP上传线程")


def __getFtp(id):
    """
    根据ID获取FTP
    :param id:      唯一标识
    :return:        FTP客户端
    """
    global __ftpClients

    for ftpClient in __ftpClients:
        if ftpClient.get("id") == id:
            return ftpClient.get("client")

    return None


def __download(ftp, remotePath, localPath, options={}, blockSize=1024):
    """
    下载数据
    :param ftp:              ftp连接对象
    :param remotePath:       远程路径
    :param localPath:        本地路径
    :return:
             True            成功
             False           失败
    """
    if ftp.sock is None:
        ftp.connect(ftp.host, ftp.port)

    fileSize = ftp.size(remotePath)
    if fileSize == 0:
        logger.info('远程文件为空文件')
        options.get("fail") and options.get("fail")()
        options.get("complete") and options.get("complete")()
        return False

    localSize = 0
    if os.path.exists(localPath):
        localSize = os.stat(localPath).st_size

    # 如果本地文件大小比远程文件大，做覆盖处理，本地文件清空
    if localSize >= fileSize:
        fileCur = open(localPath, "r+")
        fileCur.truncate()
        fileCur.close()
        logger.info("本地文件异常，做清空处理")

    cmpSize = localSize

    # 增加重试3次机制，重试3次仍然失败，报错
    connect = None

    try :
        connect = ftp.transfercmd('RETR '+remotePath, localSize)
    except Exception as e:
        try:
            time.sleep(1)
            connect = ftp.transfercmd('RETR '+remotePath, localSize)
        except Exception as e:
            try:
                time.sleep(1)
                connect = ftp.transfercmd('RETR '+remotePath, localSize)
            except Exception as e:
                options.get("fail") and options.get("fail")()
                options.get("complete") and options.get("complete")()
                logger.error(e)
                return

    dirname = os.path.dirname(localPath)
    if os.path.exists(dirname) is False:
        os.makedirs(dirname)
    fileStream = open(localPath, 'ab')
    logger.info("开始进行文件传输")

    process = 1
    interval = 0
    if fileSize < 100000000:
        interval = 10
    else:
        interval = 5
    step = fileSize / interval

    try :
        while True:
            data = connect.recv(blockSize)
            if not data:
                break
            fileStream.write(data)
            cmpSize += len(data)

            if cmpSize / step >= process:
                logger.info("下载进度: {} %".format(process*interval))
                process = process + 1
    except Exception as e:
        logger.error(e)
        options.get("fail") and options.get("fail")()
        options.get("complete") and options.get("complete")()

    fileStream.close()
    connect.close()
    ftp.voidcmd('NOOP')
    ftp.voidresp()
    logger.info("文件传输完成")

    options.get("success") and options.get("success")(localPath)
    options.get("complete") and options.get("complete")()

    return True


def __upload(ftp, localPath, remotePath, options={}, blockSize=1024):
    """
    完成上载数据操作
    :param remotePath:     远程路径
    :param localPath:      本地路径
    :return:
            True           成功
            False          失败
    """
    if not os.path.exists(localPath):
        logger.info("本地文件不存在")
        options.get("fail") and options.get("fail")()
        options.get("complete") and options.get("complete")()
        return False

    # 解析远程目录和远程文件
    remoteDir = ''
    remoteFile = ''
    remoteSize = 0
    position = remotePath.rfind('/')
    if position > 0:
        remoteDir = remotePath[:position+1]
        remoteFile = remotePath[position+1:]
    elif position == 0:
        remoteDir = '/'
        remoteFile = remotePath[position+1:]

    # 切换到远程目录
    ftp.cwd(remoteDir)

    # 此处存在远程目录为空的情况，此时size为0，需要加try
    try:
        remoteSize = ftp.size(remoteFile)
    except Exception:
        pass

    localSize = os.stat(localPath).st_size
    if remoteSize == localSize:
        logger.info("远程目录文件和本地文件大小相同，判断为同一文件")
        options.get("success") and options.get("success")(remotePath)
        options.get("complete") and options.get("complete")()
        return True

    if remoteSize < localSize:
        localFile = open(localPath,'rb')
        localFile.seek(remoteSize)

        logger.info("开始文件传输")
        connect = None
        esize = None

        try:
            connect, esize = ftp.ntransfercmd("STOR " + remoteFile, remoteSize)
        except:
            try:
                time.sleep(1)
                connect, esize = ftp.ntransfercmd("STOR " + remoteFile, remoteSize)
            except:
                try:
                    time.sleep(1)
                    connect, esize = ftp.ntransfercmd("STOR " + remoteFile, remoteSize)
                except Exception as e:
                    localFile.close()
                    logger.error(e)
                    options.get("fail") and options.get("fail")()
                    options.get("complete") and options.get("complete")()
                    return

        process = 1
        cmpSize = remoteSize
        interval = 0
        if localSize < 100000000:
            interval = 10
        else:
            interval = 5
        step = localSize / interval

        try:
            while True:
                buf = localFile.read(blockSize)
                if not len(buf):
                    break
                connect.sendall(buf)

                cmpSize += len(buf)
                if cmpSize == localSize:
                    logger.info("上传进度: 100%")
                    break
                elif cmpSize / step >= process:
                    logger.info("上传进度: {}%".format(process*interval))
                    process = process + 1
        except Exception as e:
            options.get("fail") and options.get("fail")()
            options.get("complete") and options.get("complete")()
            logger.error(e)

        connect.close()
        localFile.close()
        ftp.voidcmd('NOOP')
        ftp.voidresp()
        logger.info("文件传输完成")

    options.get("success") and options.get("success")(remotePath)
    options.get("complete") and options.get("complete")()

    return True


