# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2020, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Levi
# @version  1.0.0
# @brief    gcs数据上传
# @address  官网：      https://cloud.google.com/storage/docs/apis
#           API网站：   https://googleapis.dev/python/storage/latest/client.html
#           Google Cloud Storage API client library：https://pypi.org/project/google-cloud-storage/
#           GitHub:    https://github.com/GoogleCloudPlatform/python-docs-samples
#
#  History:
#      <author>     <time>     <version>   <desc>
#       Levi     2010-04-27     1.0.0      gcs数据上传
#############################################################
import logging

__version__ = "1.0.0"


import os
import time
import threading

import ee
from ee.batch import Export
from google.cloud import storage


__all__ = [
    'gcs2gee',
    'gee2gcs',
    'image2gcs',

    'upload',
    'download',

    'delete',
]


"""
# 常量
"""
MAX_TIMEOUT_SECONDS = 36000


"""
# 枚举
"""
class TaskStatus(object):
    READY         = 'READY'      # 准备
    RUNNING       = 'RUNNING'    # 运行中
    COMPLETED     = 'COMPLETED'  # 完成
    FAILED        = 'FAILED'     # 失败
    CANCELLED     = 'CANCELLED'  # 取消
    UNKNOWN       = 'UNKNOWN'    # 不存在此任务


"""
# 全局变量
"""
logger = logging.getLogger(__name__)


def gcs2gee(gcsPath, geePath, options={}):
    """
    GCS数据传输到GEE
    :param geePath:                GEE中Assets路径， e.g. users/yemin/image/data
    :param gcsPath:                GCS中数据存储路径，e.g. gs://data/wuhan.tif
    :param options:
                bauds:             波段列表，选择需要传输的波段名称列表，默认为空，全部传输
                success:
                    geePath        成功后返回目标路径
                fail:
                complete:
    :return:
    """
    # 准备参数
    params = {
        'id': geePath,
        'tilesets': [{
            'sources': [{
                'primaryPath': gcsPath
            }]
        }],
        'bands': options.get('bands')
    }

    # 创建任务
    newId = ee.data.newTaskId()[0]
    result = ee.data.startIngestion(newId, params)
    logger.info("创建任务成功，ID: {}".format(newId))

    # 建立线程，监听任务完成
    thread = threading.Thread(target=__checkTaskStatus,
                              args=(result.get('id'), options, geePath))
    thread.start()
    logger.info("开始任务")


def gcs2gee2(bucket, path, geePath, options={}):
    """
    GCS数据传输到GEE,重载方法
    :param bucket                  GCS数据存储桶名称
    :param path                    存储桶中数据路径
    :param geePath:                GEE中Assets路径， e.g. users/yemin/image/data
    :param options:
                success:
                    geePath        成功后返回目标路径
                fail:
                complete:
    :return:
    """
    gcsPath = 'gs://' + bucket + '/' + path
    gcs2gee(gcsPath, geePath, options)


def gee2gcs(geePath, bucket, path, options={}):
    """
    GEE数据传输到GCS
    :param geePath:               GEE中Assets路径， e.g. users/yemin/image/data
    :param bucket:                GCS数据存储桶名称
    :param path:                  存储桶中数据路径
    :param options:
                success:
                    bucket+path   成功后返回目标路径
                fail:
                complete:
    :return:
    """
    image = ee.Image(geePath)
    newTask = Export.image.toCloudStorage(image, bucket=bucket, fileNamePrefix=path, maxPixels=1e13)
    newTask.start()
    logger.info("启动数据传输任务，ID: {}".format(newTask.id))

    gcsPath = None
    if path.startswith('/'):
        gcsPath = bucket + path
    else:
        gcsPath = bucket + '/' + path

    # 开启线程，监听任务完成
    thread = threading.Thread(target=__checkTaskStatus,
                              args=(newTask.id, options, gcsPath))
    thread.start()


def gee2gcs2(geePath, gcsPath, options={}):
    """
    GEE数据传输到GCS
    :param geePath:              GEE中Assets路径， e.g. users/yemin/image/data
    :param gcsPath:              GCS中数据存储路径，e.g. gs://data/wuhan.tif
    :param options:
                bauds:
                success:
                    gcsPath      成功后返回目标路径
                fail:
                success:
    :return:
    """
    gsIndex = gcsPath.index('gs://')
    tempStr = gcsPath[gsIndex+1:]

    slashIndex = tempStr.index('/')
    bucket = tempStr[0:slashIndex]
    path = tempStr[slashIndex+1:]

    gee2gcs(geePath, bucket, path, options)


def image2gcs(image, bucket, path, options={}):
    """
    影像对象上传到GCS存储桶
    :param image:               影像对象
    :param bucket:              存储桶名称
    :param path:                存储路径
    :param options:
               epsg:            EPSG代码，例如EPSG:3857
               success:
                   bucketPath   存储路径
                fail:
                success:
    :return:
    """
    description = 'myExportImageTask'
    if options.get('description') is not None:
        description = options.get('description')
    newTask = None
    if options.get('epsg') is not None:
        newTask = Export.image.toCloudStorage(image, description=description, bucket=bucket, fileNamePrefix=path,
                                              maxPixels=1e13, crs=options.get('epsg'),
                                              region=options.get('region'),
                                              scale=options.get('scale'))
    else:
        newTask = Export.image.toCloudStorage(image, description=description, bucket=bucket, fileNamePrefix=path,
                                              maxPixels=1e13, crs=options.get('crs'),
                                              region=options.get('region'), scale=options.get('scale'))
    if newTask is not None:
        newTask.start()
        logger.info("启动数据传输任务，ID: {}".format(newTask.id))

        # 开启线程，监听任务完成
        thread = threading.Thread(target=__checkTaskStatus,
                                  args=(newTask.id, options, bucket+'/'+path))
        thread.start()


def upload(localPath, bucket, path, options={}):
    """
    从本地磁盘上载到GCS中
    :param localPath:      本地路径
    :param bucket:         存储桶名称
    :param path:           存储路径
    :param options:
               success
               fail
               complete
    :return:
    """
    client = storage.Client()
    bucket = client.bucket(bucket)
    if bucket is not None:
        blob = bucket.blob(blob_name=path)

        thread = threading.Thread(target=__uploadThread, args=(blob, localPath, options))
        thread.start()
        logger.info("开启数据上传线程")


def download(bucket, path, localPath, options={}):
    """
    从GCS中下载数据到本地磁盘
    API参考网站：https://googleapis.dev/python/storage/latest/client.html
    :param bucket:          存储桶
    :param path:            存储路径
    :param localPath:       本地文件路径
    :param options:
               success
                   path     目标路径
               fail
               complete
    :return:
    """
    client = storage.Client()
    bucket = client.get_bucket(bucket)
    blob = bucket.blob(path)
    if blob is not None:
        thread = threading.Thread(target=__downloadThread, args=(blob, localPath, options))
        thread.start()
        logger.info("开启数据下载线程")


def delete(bucket, path, options={}):
    """
    删除指定存储桶的数据
    :param bucket:       存储桶名称
    :param path:         数据路径
    :param options:
               success
               fail
               complete
    :return:
    """
    client = storage.Client()
    bucket = client.get_bucket(bucket)
    if bucket is not None:
        blob = bucket.blob(path)
        if blob.exists() is True:
            blob.delete()
            logger.info("数据删除成功")
    else:
        options.get('fail') and options.get('fail')()
        options.get('complete') and options.get('complete')()

    options.get('success') and options.get('success')()
    options.get('complete') and options.get('complete')()


def __checkTaskStatus(taskId, options={}, successArg=None):
    """
    检查任务的状态
    :param taskId:           任务ID
    :param options:
               success
               fail
               complete
    :param successArg:      成功后结果参数
    :return:
    """
    num = 0
    while True:
        num += 1
        # 如果超过10个小时还未完成，跳出循环
        if num > MAX_TIMEOUT_SECONDS:
            ee.data.cancelTask(taskId)
            options.get('fail')  and options.get('fail')()
            options.get('complete') and options.get('complete')()
            logger.info("传输task超过限定的最大时间，终止任务")
            break

        taskStatuses = ee.data.getTaskStatus(taskId)
        if len(taskStatuses) > 0:
            status = taskStatuses[0].get('state')
            if status == TaskStatus.UNKNOWN or status == TaskStatus.CANCELLED or status == TaskStatus.FAILED:
                options.get('fail')     and options.get('fail')()
                options.get('complete') and options.get('complete')()
                logger.info('任务{}执行失败'.format(taskId))
                break
            elif status == TaskStatus.COMPLETED:
                options.get('success')  and options.get('success')(successArg)
                options.get('complete') and options.get('complete')()
                logger.info('任务{}执行成功'.format(taskId))
                break
        else:
            options.get('fail')     and options.get('fail')()
            options.get('complete') and options.get('complete')()
            logger.info('任务{}执行失败'.format(taskId))
            break
            
        # 休眠15s
        time.sleep(15)


def __downloadThread(blob, path, options):
    """
    从GCS下载数据
    :param blob:        区块数据对象
    :param path:        数据存储路径
    :param options:
               success
                   path 目标路径
               fail
               complete
    :return:
    """
    dir = os.path.dirname(path)
    if os.path.exists(dir) is False:
        os.makedirs(dir)

    try:
        blob.download_to_filename(filename=path, checksum=None)
        options.get("success") and options.get("success")(path)
        logger.info("{} 下载完成".format(path))
    except Exception as e:
        logger.error(e)
        try:
            blob.download_to_filename(filename=path, checksum=None)
            options.get("success") and options.get("success")(path)
            logger.info("{} 下载完成".format(path))
        except Exception as e:
            logger.error(e)
            options.get("fail") and options.get("fail")()


def __uploadThread(blob, path, options):
    """
    将本地数据上传到GCS
    :param blob:         区块数据
    :param path:         存储路径
    :param options:
               success
               fail
               complete
    :return:
    """
    try:
        blob.upload_from_filename(filename=path)
        options.get("success") and options.get("success")(path)
        logger.info("{} 上传完成".format(path))
    except Exception as e:
        logger.error(e)
        try:
            blob.upload_from_filename(filename=path)
            options.get("success") and options.get("success")(path)
            logger.info("{} 上传完成".format(path))
        except Exception as e:
            logger.error(e)
            options.get("fail") and options.get("fail")()





