# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2020, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Eason
# @version  1.0.0
# @brief    MQTT消息通信
# @address  MQTT工程: https://github.com/eclipse/paho.mqtt.python
#
#  History:
#      <author>     <time>     <version>   <desc>
#       Eason      2010-05-13     1.0.0   MQTT消息通信
#############################################################

import logging
import socket
import time
import hashlib
import threading
import json

import paho.mqtt.client as mqtt
import hrse.hrandom as hrandom

logger = logging.getLogger(__name__)


__version__ = "1.0.0"


__all__ = [
    "init",
    "send",
    "register",
    "unregister",
    "getClinet",
]


"""
# 常量
"""
MQTT_PASSWORD_KEY = "Ox7^3k*B"


"""
# 枚举
"""


"""
# 全局变量
"""
# MQTT连接
__mqttClient = None

# 消息回调列表
__messageCallbacks = []


def init(clientid=None, username=None, password=None, options={}):
    """
    初始化
    :param clientid: 终端id，为空时自动使用随机数
    :param username: 用户名
    :param password: 用户密码
    :param options:
                host: 域名
                port: 端口
                keepalive: 保活
    :return:
    """
    global __mqttClient

    options.setdefault("host", "mqtt.hedadata.com")
    options.setdefault("port", 1783)
    options.setdefault("keepalive", 60)

    # 断开旧的连接
    if __mqttClient:
        __mqttClient.disconnect()

    # 默认clientid
    if not clientid:
        clientid = "py_{}".format(hrandom.generate(8))

    __mqttClient = mqtt.Client(client_id=clientid)
    __mqttClient.on_message = __on_message
    __mqttClient.on_connect = __on_connect
    __mqttClient.on_log = __on_log

    # 默认密码：MD5(username + KEY)前8个字符
    if username:
        if not password:
            md5 = hashlib.md5()
            md5.update((username + MQTT_PASSWORD_KEY).encode("utf-8"))
            password = md5.hexdigest()[0: 8]

        __mqttClient.username_pw_set(username, password)

    # 连接
    __mqttClient.connect(options.get("host"), options.get("port"), options.get("keepalive"))

    # 在独立线程中运行，不堵塞主线程
    def threadRun():
        __mqttClient.loop_forever()

    thread = threading.Thread(target=threadRun)
    thread.start()


# 连接信息回调
def __on_connect(mqttc, obj, flags, rc):
    logger.info("MQTT连接：" + mqtt.connack_string(rc))


# 收到推送消息
def __on_message(mqttc, obj, msg):
    global __messageCallbacks

    logger.info("MQTT收到消息：{} {} {}".format(msg.topic, str(msg.qos), str(msg.payload)))

    for messageCallback in __messageCallbacks:
        if messageCallback.get('topic') == msg.topic:
            messageCallback.get('callback')(msg)


# 日志
def __on_log(mqttc, obj, level, string):
    logger.debug(string)


def send(topic, payload=None, qos=2, retain=False):
    """
    发送数据
    :param topic: 主题
    :param payload: 数据
    :param qos: 通信指令
    :param retain:
    :return:
    """
    if __mqttClient:
        __mqttClient.publish(topic, payload, qos, retain)


def register(topic, callback):
    """
    注册回调
    :param topic: 主题
    :param callback: 回调
                msg: 消息对象
                    topic: 主题
                    qos: 质量
                    payload: 数据
    :return: 取消注册方法
    """
    global __mqttClient
    global __messageCallbacks

    # 如果是新的主题，需要发起订阅
    isNewTopic = True
    for messageCallback in __messageCallbacks:
        if messageCallback.get('topic') == topic:
            isNewTopic = False
            break

    if isNewTopic:
        # TODO: 目前统一使用qos为2
        __mqttClient.subscribe(topic, 2)

    __messageCallbacks.append({
        'topic': topic,
        'callback': callback
    })

    def _unregister():
        unregister(topic=topic, callback=callback)

    return _unregister


def unregister(topic, callback):
    """
    取消注册回调
    :param topic: 主题
    :param callback: 回调
    :return:
    """
    global __messageCallbacks

    for i in range(len(__messageCallbacks) - 1, 0, -1):
        if topic == __messageCallbacks[i].get('topic') and callback == __messageCallbacks[i].get('callback'):
            del __messageCallbacks[i]


def getClinet():
    """
    获取mqtt客户端
    :return:
    """
    return __mqttClient


if __name__ == "__main__":
    # deviceid = "hrs0001"
    # clientid = "o_{}".format(hrandom.generate(12))
    # username = "tx2_o_n_{}_{}".format(deviceid, hrandom.generate(4))
    #
    # init(clientid=clientid, username=username)
    #
    # def func(msg):
    #     logger.error("收到消息：{}".format(str(msg)))
    #     send("/hrs/server", json.dumps({
    #         "ii": 8
    #     }))
    #
    # register("/hrs/usserver", func)
    #
    # while(True):
    #     time.sleep(1)

    pass
