# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2019, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Eason Liu
# @version  1.0.0
# @brief    模板模块
#           https://kazoo.readthedocs.io/en/latest/
#  History:
#      <author>     <time>     <version>   <desc>
#       Eason     2019-11-22     1.0.0     初始化
#############################################################
import logging
import urllib
from urllib.parse import quote_plus
from kazoo.client import KazooClient


__version__ = "1.0.0"


__all__ = [
    'start',
    'stop',

    'getDubboServices',
    'deleteDubboService',

    'registerStatus',
    'unregisterStatus',
]


"""
# 常量
"""


"""
# 枚举
"""


"""
# 全局变量
"""
# 日志管理对象
logger = logging.getLogger(__name__)

# Kazoo客户端
__kazooClient = None

# 回调函数数组
__statusCallbacks = []


def __kazooListener(state):
    """
    Kazoo监听器
    :param state: KazooState
    :return:
    """
    global __kazooClient
    global __statusCallbacks

    # 处理断线重连
    if state == "LOST":
        pass
        # Register somewhere that the session was lost
        # __kazooClient.restart()
    elif state == "SUSPENDED":
        # Handle being disconnected from Zookeeper
        __kazooClient.restart()
    else:
        # Handle being connected/reconnected to Zookeeper
        pass

    # 执行回调
    for callback in __statusCallbacks:
        callback(state)


def start(host, port, options={}):
    """
    启动Zookeeper
    1. 创建并配置KazooClient对象
    2. 如果上一个对象非空，需要先关闭后，再创建新对象（如果Client可以再次配置(set_hosts)，就只需要再次配置即可）
    3. 未来需要做安全认证
    :param host: Zookeeper中心地址
    :param port: 中心端口
    :param options:
               success
               fail
               complete
    :return: None
    """

    global __kazooClient

    hosts = host + ":" + port
    if __kazooClient is not None:
        if __kazooClient.hosts is not hosts:
            __kazooClient.set_hosts(hosts)
            __kazooClient.restart()
    else:
        __kazooClient = KazooClient(hosts)
        __kazooClient.add_listener(__kazooListener)

        __kazooClient.start()

    options.get("success")  and options.success()
    options.get("complete") and options.complete()


def stop():
    """
    关闭Zookeeper
    :return: None
    """
    global __kazooClient
    if __kazooClient is not None:
        __kazooClient.stop()
        __kazooClient.close()
        __kazooClient = None


def getDubboServices(serviceName):
    """
    获取Dubbo服务列表
    :param serviceName: 服务名称，支持模糊匹配
    :return: services
                service: 服务名称
                paths: 路径列表
                methods: 方法列表
    """
    global __kazooClient
    services = None
    if __kazooClient is not None:
        if __kazooClient.connected is False:
            __kazooClient.restart()
        urls = []
        dubbos = __kazooClient.get_children("dubbo")
        for dubbo in dubbos:
            if serviceName in dubbo:
                # 获取服务发布方
                providers = __kazooClient.get_children("/dubbo/{}/providers".format(dubbo))
                if providers:
                    for provider in providers:
                        url = urllib.parse.unquote(provider)
                        if url.startswith('dubbo:'):
                            urls.append(url.split('dubbo://')[1])

        paths = []
        methods = []
        service = None
        for url in urls:
            path, temp = url.split('/')
            service = temp.split('?')[0]
            methods = temp.split('methods=')[1].split('&')[0].split(',')
            if paths.__contains__(path):
                pass
            else:
                paths.append(path)

        if service is not None:
            services = {
                "service": service,
                "paths": paths,
                "methods": methods
            }
    return services


def __getProviderPath(serviceName):
    global __kazooClient
    if __kazooClient is None:
        return

    urls = []
    dubbos = __kazooClient.get_children("dubbo")
    for dubbo in dubbos:
        if serviceName in dubbo:
            providers = __kazooClient.get_children("/dubbo/{}/providers".format(dubbo))
            if providers is not None:
                for provider in providers:
                    url = urllib.parse.unquote(provider)
                    urls.append("/dubbo/{}/providers".format(dubbo) + '/' + url)
    return urls

def deleteDubboService(serviceName, port):
    """
    zookeeper中心删除服务
    :param serviceName: 服务路径
    :param port:        端口号
    :return:
    """
    global __kazooClient
    if __kazooClient is None:
        return

    dubbos = __kazooClient.get_children("dubbo")
    for dubbo in dubbos:
        if serviceName in dubbo:
            providers = __kazooClient.get_children("/dubbo/{}/providers".format(dubbo))
            if len(providers) > 0:
                for provider in providers:
                    url = urllib.parse.unquote(provider)
                    if str(port) in url:
                        __kazooClient.delete("/dubbo/{}/providers".format(dubbo) + '/' + quote_plus(url))


def registerStatus(callback):
    """
    注册监听状态事件
    :param listener 监听函数
    :param callback: 监听回调
               oldState: ZookeeperStatus, 旧状态
               newState: ZookeeperStatus, 新状态
    :return: 取消监听方法
    """
    global __statusCallbacks

    if callback not in __statusCallbacks:
        __statusCallbacks.append(callback)

    def unregisterCallback():
        unregisterStatus(callback)

    return unregisterCallback


def unregisterStatus(callback):
    """
    取消监听
    :param callback: 监听回调
    :return: None
    """
    global __statusCallbacks

    __statusCallbacks.remove(callback)


def registerNodeChange(callback):
    """
    注册监听节点变更事件
    TODO: 尚未实现本功能，可参考：https://kazoo.readthedocs.io/en/latest/basic_usage.html?highlight=node#watchers
    :param callback: 监听回调
    :return: 取消监听方法
    """
    pass


if __name__ == "__main__":
    pass
